****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidacov.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 06/30/2019
* Version: 3.5
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will manage covariates.                                        
*   
*  Program inputs:  
*	-DPLocal.&RUNID._&data.
*	-infolder.&COHORTFILE. 
*	-infolder.&UTILFILE.
*	-infolder.Comorbcodes
*	-infolder.&Covariatecodes.
*	-infolder.&DRUGCLASSFILE.
*	-infolder.&MONITORINGFILE.
*                                     
*  Program outputs: 
*	-DPLocal.&RUNID._ads_&data.
*	-DPLocal.&RUNID._Claims_icddx09
*	-DPLocal.&RUNID._Claims_icddx10
*	-DPLocal.&RUNID._Claims_icddxOT
*	-DPLocal.&RUNID._Claims_icdpx09
*	-DPLocal.&RUNID._Claims_icdpx10
*	-DPLocal.&RUNID._Claims_icdpxOT
*	-DPLocal.&RUNID._Claims_cpt
*	-DPLocal.&RUNID._Claims_hcpcs
*	-DPLocal.&RUNID._Claims_drugclass 
*	-DPLocal.&RUNID._Claims_rx
*	-DPLocal.&RUNID._Claims_lab   
*	-MSOC.&RUNID._baseline&outcohort._&i.;
*	-MSOC.&RUNID._profile._&i.;
*	-MSOC.&RUNID._baseline_&mfu._&i.;

*  PARAMETERS:       
*	-data: indicates input the patient-level dataset  
*	-outcohort: indicates the suffix for output baseline table if additional cohorts are created
*            
*  Programming Notes:                                                                                
*	-This macro includes the optional execution of ms_util and ms_cci_elix macros                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*     1.3    03/31/2015    AC       Made changes to programming notes in header section.
*
*     1.4    03/31/2015    EM       Adapt code to Surveilance mode.
*
*     1.5    06/30/2015    EM       Removed Surveillance Analysis related code
*
*     1.6    06/30/2015    EM       Calculation of a baseline table (similar to table 1) for TYPE1 and TYPE2 requests.  
*
*     1.7    12/23/2015    EM       Changed age group names to be consitent with table 1.  
*
*     1.8    01/15/2016    VC       Allow for greater flexibility specifying the covariate assessment window.  
*									Create combined covariate variables.
*
*	  1.9    04/21/2016	   DM  		Added a w.a.r.n.i.n.g. when no baseline table is produced because no 
*									users were identified in the request
*
*	  1.10   08/10/2016	  AP(SOC)	Added 2 new HDPS dimension files (DX10 and PX10) for ICD10 (QCI-186)
*
*	  1.11   08/15/2016    DM       Added Surveillance Analysis related code removed in 1.5
*
*	  1.12   10/17/2016    DM       Fixed a couple of issues regarding Surveillance Analysis
*
*	  1.13   11/30/2016	  AP		Removed infolder libname from Monitoring File sort to accomodate 
*									truncation by DPMax Date	
*
*	  1.14   12/09/2016	  AP		Remove KeepAllDum as a parameter in the ComorbFile
*
*	  1.15   12/12/2016	  AP		Removed code that de-duped RX codes across covariates (QRP-289)
*
*	  2.0    03/27/2017	  DM		Removed covariates extraction from CDM. Now done earlier in ms_cidanum 
*
*	  2.1    05/05/2017	  AP		Added race, hispanic, and year to baseline table
*
*	  2.2	 07/10/2017	  AP		1. Added baseline table for Types 3-5
*									2. Added covariate profile table
*									3. Added MFU analysis
*
*	  2.3	 09/14/2017	  AP		Bug fix (QRP-429)
*
*	  2.4	 09/27/2017	  AP		Changed Type 3 dataset names, MFU lab extraction (QRP-432, QRP-433)
*
*	  2.5	 11/15/2017	  AP		Added shaving of covariates (QRP-419)
*
*	  2.6	 12/07/2017	  AP		1. Fixed PX table name in CCI macro call (QRP-469)
*									2. Sorting T4 control group by delnum (QRP-471)
*
*	  2.7	 12/21/2017	  AP		1. Modified comorbidity score window parameters (QRP-435)
*									2. Added CODEDAYS to covariate selection (QRP-455)
*
*	  2.8    02/27/2018	  RR		1. Ability to add never exposed group if requested (QRP-495)
*
*	  2.9    03/27/2018	  AP		Removed group variable from ms_util hash table (QRP-534)
*
*	  2.10   04/05/2018	  AP		Modified MedUtil parameters (QRP-460)
*
*     2.11   08/06/2018	  RR        Bug Fix: Fix Type 4 baseline table query period bug 
*
*	  3.0    05/20/2018	  AP		Separated baseline table, MFU and HDPS table extract to separate macros
*									Added ability to proudce baseline table for analysis groups
*
*	  3.1	 10/25/2018	  AP		Moved create claims file and MFU out of macro into cidanum
*
*	  3.2	 11/12/2018	  AP		Modified for conditional enrollment extraction and saved some files to DPLOCAL
*
*	  3.3	 12/18/2018	  AP		Added combo RX code extraction for covariate codes
*
*	  3.4	 02/28/2019	  AP		Implemented codesupply for covariatecodes
*
*	  3.5	 06/30/2019	  AP		Compatibility with QRP 8.0.0 updates
*
***************************************************************************************************;

%macro ms_cidacov(data=, outcohort=, groupvar=, groupdefdata=);

%put =====> MACRO CALLED: ms_cidacov v3.5;

	%ms_starttimer(timestart=cidacovruntime);
    
	/******************/
	/* Pre-Processing */
	/******************/

	*List of patients from CIDANum;

    proc sort nodupkey data=DPLocal.&RUNID._&data. out=PtsList(keep=Patid);
    	by Patid;
    run;

    proc sort nodupkey data=DPLocal.&RUNID._&data. out=&RUNID._&data.;
    	by &groupvar. Patid IndexDt;
    run;

	*Initialize date variable for creating individual PeriodId files;
	%if %eval(&type. = 4) & "&data" ne "mstr_mi" %then %do;
		%let lookdt = EpisodeEndDt;
	%end;
	%else %do;
		%let lookdt = IndexDt;
	%end;

	*must have at least one patient identified in the run;
	%isdata(dataset=&RUNID._&data.);
	%if %eval(&NOBS. > 0) %then %do;

	*Assign pointer to prior DPLocal when running surveillance mode;
	%global DPLOCALPOINTER;
	%let DPLOCALPOINTER=DPLocal;
	%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %let DPLocalPointer=DPLPrior;
	%put NOTE: DPLOCAL for prior run is set to &DPLocalPointer.;	

	*Creating the number of groups to create cohorts;
	 %isdata(dataset=&groupdefdata.);    
	 data _null_;
	 	call symputx("Numgroup","&nobs.");
	 run;

	 %if "&groupvar" = "group" %then %let groupvar2 = cohortgrp;
	 	%else %let groupvar2 = &groupvar.;

	*Initialize the number of groups;
	%let Ngrp = 1;

	/*********************************/
	/* Compute Utilization Variables */
	/*********************************/ 

    %macro RunUtil;

	    %ISDATA(dataset=UTILFILE);	
	    %IF %EVAL(&NOBS.>0) %THEN %DO;

	        *Extracting Encounter Data;
	        data dplocal._baseenc(keep=PatId ADate EncType);
	        if 0 then set PtsList;
	        declare hash ht (hashexp:16, dataset:"PtsList");
	        ht.definekey('PatId');
	        ht.definedone();

	        do until(eof1);
	          set &datalib_enc. end=eof1;
	          if ht.find()=0 then output;
	        end;
	        stop;
	        run;

	        %do Group=1 %to &Numgroup.;

				*Select group;
				data _null_;
					set &groupdefdata.;
					if _n_=&group. then do;
		            call symputx('ITGROUP',strip(&groupvar2.));
					end;
				run;

	            *Processing Util parameters;
	            data _ITUtil;
		            set UTILFILE;
		            where group = "&itgroup.";
		            call symputx("MEDUTILFROM",coalesce(MEDUTILFROM, -99999));
		        	call symputx("MEDUTILTO",coalesce(MEDUTILTO, 99999));
		        	*call symputx("CARESETTINGS",compress(CARESETTINGS,","));
		        	call symputx("CSSTRAT",CSSTRAT);
		            call symputx("CARESETTINGS","'AV' 'OA' 'IP' 'IS' 'ED'");  *defensive coding; 
	            run;

		       *Creating index reference file;
				proc sort data=&RUNID._&data. out= _indexfile(keep=patid indexdt) %if %eval(&type.=5) %then %do; nodupkey %end; ;
					by patid &groupvar. indexdt;
					where &groupvar. = "&ITGROUP.";
				run;

				%isdata(dataset=_ITUtil);
				%if %eval(&nobs.>0) %then %do;

		            *Calculate Utilization;
		            %MS_UTIL(ENCOUNTERFILE=dplocal._baseenc,		
		              	     INDEXFILE=_IndexFile,
		            	     INDEXDT=IndexDt, 
		            	     UTILFROM=&MEDUTILFROM.,
		            	     UTILTO=&MEDUTILTO.,
		            	     CARESETTINGS=&CARESETTINGS., 
		            	     BYENC=Y, 					/*QRP will default to Y*/
		            	     CSSTRAT=&CSSTRAT.,
		            	     OUTFILE=_ITUtil 
		                 );  

		            *Post Process;
		            data _ITUtil;
			            set _ITUtil;
			            format &groupvar. $40.;
			            &groupvar.="&ITGROUP."; 
			            length NumVisits $10.;
			            format NumVisits $10.;
			            NumVisits=NumVisit;
			            drop NumVisit;

						rename 	NumVisits_AV=NumAV
								NumVisits_IP=NumIP
								NumVisits_ED=NumED
								NumVisits_OA=NumOA
								NumVisits_IS=NumIS;
		            run;
				%end;
				%else %do;
  					data _ITUtil;
						set _IndexFile(obs=0);
			            format &groupvar. $40.;
			            &groupvar.="&ITGROUP."; 
			            length NumVisits $10.;
			            format NumVisits $10.;
							ExactNumVisit=.;
							NumAV=.;
							NumIP=.;
							NumED=.;
							NumOA=.;
							NumIS=.;
							NumVisits="";
		            run;				
				%end;

	            *Combine Util for multiple groups;
	            %IF %EVAL(&group.=1) %THEN %DO;
	                data _AllUtil;
	                set _ITUtil;
	                run;
	            %END;
	            %ELSE %DO;
	                proc append base=_AllUtil
	                            data=_ITUtil force;
	                run;
	            %END;

	            *Clean-up;
	            proc datasets library=work nolist nowarn;
	            delete _ITUtil _IndexFile;
	            quit;

		        proc sort data=_AllUtil;
		        by &groupvar. PatId IndexDt;
		        run;
	     	%END; *do loop;

			proc datasets nowarn nolist lib=dplocal;
				delete _baseenc;
			quit;

			%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
			    data _NewPatients;
			    set DPLocal.&RUNID._&data.;
			    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
			    keep &groupvar. PatId IndexDt;
			    run;

			    proc sort nodupkey;
			    by &groupvar. PatId IndexDt;
			    run;

			    data _AllUtil;
			    merge _AllUtil(in=a)
			          _NewPatients(in=b);
			    by &groupvar. PatId IndexDt;
			    if a and b;
			    run;
			%end;

	        data DPLocal.&RUNID._&data.;
	        merge DPLocal.&RUNID._&data.
	              _AllUtil;
	        by &groupvar. PatId IndexDt;
	        run;

	        %ISDATA(dataset=infolder.&DRUGCLASSFILE.);	
	        %IF %EVAL(&NOBS.>0) %THEN %DO;

				proc sort data=&RUNID._&data. out= _BaseTmp(keep=&groupvar. patid indexdt) %if %eval(&type.=5) %then %do; nodupkey %end; ;
					by &groupvar. patid indexdt;
				run;

				data utilhash;
					set UTILFILE(Keep=group DrugUtilFrom DrugUtilTo 
						%if "&groupvar" = "analysisgrp" %then %do; rename=group=&groupvar. %end;);
				run;

	    	    data _BaseTmp;
	    	    if 0 then set utilhash;
	    	    declare hash ht (hashexp:16, dataset:"UTILHASH");
	    	    ht.definekey("&groupvar.");
	    	    ht.definedata('DrugUtilFrom','DrugUtilTo'); 

	    	    ht.definedone();

	    	    do until(eof1);
	    	    set _BaseTmp end=eof1;
	        	    if ht.find()=0 then do;
	                if .<DrugUtilFrom < 0 then DrugUtilFrom=-1*DrugUtilFrom;                
	                output;
	                end;
	    	    end;
	    	    stop;
	    	    run;

	            *calculate NumGeneric-NumClass;
	            *extract Rx data in covariatewindow before IndexDt;
	            proc sql noprint;
	            create table _UtilRx as
	            select Pts.&groupvar., 
	                   Pts.PatId,
	                   Pts.IndexDt,
	                   disp.NDC
	            from _BaseTmp as pts,
	                 &datalib_dis. as disp
	            where pts.PatId = disp.Patid and 
	                  %MS_PeriodsOverlap(period1=pts.IndexDt-COALESCE(pts.DrugUtilFrom,99999) COALESCE(pts.indexdt+DrugUtilTo/*-1*/,99999),                             
	                                     period2=disp.RxDate);
	            quit;

	            *get Generic ClassName;
	            proc sql noprint;
	            create table _UtilRxClassGen as
	            select Pts.&groupvar., 
	                   Pts.PatId,
	                   Pts.IndexDt,
	                   RxCl.Generic,
	                   RxCl.ClassName,
	                   1 as one
	            from _UtilRx as pts,
	                 infolder.&DRUGCLASSFILE. as RxCl
	            where pts.NDC = RxCl.ndc;
	            quit;

	            *Count number of rows in dispensing table with NDC in drugclass file ;
	            proc sql noprint;
	            create table _NumRx as
	            select Pts.&groupvar., 
	                   Pts.PatId,
	                   Pts.IndexDt,
	                   1 as one
	            from _UtilRx as pts,
	                 (select distinct ndc from infolder.&DRUGCLASSFILE.) as RxCl
	            where pts.NDC = RxCl.ndc;
	            quit;
	            
	            proc means data=_NumRx noprint nway;
	            var one;
	            class &groupvar. PatId IndexDt;
	            output out=_NumRx(drop=_:) sum=NumRx;
	            run;

	            *Count number of disctinct Generic;
	            proc sort nodupkey data=_UtilRxClassGen out=_Generic;
	            by &groupvar. PatId IndexDt Generic;
	            run;
	            proc means data=_Generic noprint;
	            var one;
	            by &groupvar. PatId IndexDt;
	            output out=_Generic(drop=_:) sum=NumGeneric;
	            run;

	            *Count number of disctinct ClassName;
	            proc sort nodupkey data=_UtilRxClassGen out=_ClassName;
	            by &groupvar. PatId IndexDt ClassName;
	            run;
	            proc means data=_ClassName noprint;
	            var one;
	            by &groupvar. PatId IndexDt;
	            output out=_ClassName(drop=_:) sum=NumClass;
	            run;

	            *combine all three metrics;
	            data _combine;
	            merge _Generic
	                  _ClassName
	                  _NumRx;
	            by &groupvar. PatId IndexDt;
	            run;

				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
				    data _combine;
				    merge _combine(in=a)
				          _NewPatients(in=b);
				    by &groupvar. PatId IndexDt;
				    if a and b;
				    run;

				    %if %eval(&PERIODIDSTART.>1) %then %do;			      	  

				      %macro PriorPeriods;
				        *complete covariates with pre-indexdt values from prior periods;
				        data _PriorPeriods;
				        set &DPLOCALPOINTER..&RUNID._ads_&data._%eval(&PERIODIDSTART.-1);
				        by &groupvar. PatId IndexDt;
				        keep &groupvar. PatId IndexDt NumGeneric NumClass NumRx;
				        run;
				      %mend;
				      %PriorPeriods;

				      data _baseTmp;
				      merge _baseTmp
				            _PriorPeriods;
				      by &groupvar. PatId IndexDt;
				      run;

				      proc datasets library=work nolist nowarn;
				      delete _PriorPeriods;
				      run;
				    %end;

				%end;

	            *put Back in _Base;        
	            data _baseTmp;
	            merge _baseTmp(in=a)
	                  _combine(keep=&groupvar. PatId IndexDt NumRx NumGeneric NumClass); *combine only contains new patients - other vars already updated;
	            by &groupvar. PatId IndexDt;
	            if a;  
	            run;

	            *Update Master file with utilization data;
	            data DPLocal.&RUNID._&data.;
	            merge DPLocal.&RUNID._&data.
	                  _basetmp;
	            by &groupvar. PatId IndexDt;
				if missing(NumRx) then NumRx=0;
				if missing(NumGeneric) then NumGeneric=0;
				if missing(NumClass) then NumClass=0;
	            run;

	            proc datasets library=work nolist nowarn;
	            delete _BaseEnc IndexFile;
	            quit;
	        %END;
	        %ISDATA(dataset=infolder.&DRUGCLASSFILE.);	
	        %IF %EVAL(&NOBS.=0) %THEN %DO;
	    		data DPLocal.&RUNID._&data.;
	    		set DPLocal.&RUNID._&data.;
	    		NumRx=.;
	    		NumGeneric=.;
	    		NumClass=.;
	    		run;
	        %END;
	    %END;  
		%ELSE %DO;		
			data DPLocal.&RUNID._&data.;
			set DPLocal.&RUNID._&data.;
			ExactNumVisit=.;
			NumAV=.;
			NumIP=.;
			NumED=.;
			NumOA=.;
			NumIS=.;
			NumVisits="";
			NumRx=.;
			NumGeneric=.;
			NumClass=.;
			run;
		%END;	*util calcs;
    %mend;
    %RunUtil;

	/*********************************************/
	/* and pre-extract the data for ADS          */ 
	/* and Charlson Comorbidity/Elixhauser Index */
	/*********************************************/

    %macro RunCovExtraction;

	    *Test whether extraction is necessary;
	    %IF %LENGTH(&CovarCodes.) | %LENGTH(&Comorbfile.) %THEN %DO;
	        * Extraction needed: Create extraction lookup files;    
	        data _CovRx
	             CovMeds;
	        set &ComborbsCodes.
	            &CovarCodes.;       
	        if CodeCat="RX" then output _CovRx; 
			&CovarCodes2.;
	        run;

		%MS_ProcessWildcards(InFile=CovMeds, 
	                         CodeVar=code, 
	                         OutFile=CovMeds);

	    *Break out CaresettingPrincipal into Caresetting and Principal;
	    %ms_caresettingprincipal(InFile=CovMeds, 
	                             Var=CareSettingPrincipal, 
	                             OutFile=CovMeds);

		*Extracting meds using EncType and PDX to meds;
	    %ms_loopmeds(datafile=dplocal.meds,
	                 lookfile=CovMeds, 
					 multiplecodecat=Y,
	                 outfile=_temp_PreMeds);

		*restricting to pts in cidanum;
	 	data _temp_PreMeds1(rename=DummyDate=ADate);
	        if 0 then set PtsList;
	        declare hash pt (hashexp:16, dataset:"PtsList");
	        pt.definekey('PatId');
	        pt.definedone();

	        do until(eof1);
	          set _temp_PreMeds end=eof1;
	          if pt.find()=0 then do;
				format DummyDate mmddyy10.;
				length DummyDate 4.;
				DummyDate=ADate;
				RxSup=1;
				RxAmt=1;
				drop ADate;
				output;
			end;
	        end;
	        stop;
		run;

	    %ms_extractdrugs(datafile=dplocal.drugs,
	                     datavar=NDC ,
	                     lookfile=_CovRx(drop=codecat CodeType), 
	                     lookvar=code, 
	                     outfile=_temp_PreDrugs);     

	    %ms_extractdrugs(datafile=dplocal.drugscomb,
	                     datavar=NDC ,
	                     lookfile=_CovRx(drop=codecat CodeType), 
	                     lookvar=code , 
	                     outfile=_temp_PreDrugsComb);

		*restricting to pts in cidanum;
	 	data _temp_PreDrugs1(rename=DummyDate=RxDate);
	        if 0 then set PtsList;
	        declare hash pt1 (hashexp:16, dataset:"PtsList");
	        pt1.definekey('PatId');
	        pt1.definedone();

	        do until(eof1);
	          set _temp_PreDrugs end=eof1;
	          if pt1.find()=0 then do;
			  	length DummyDate 4.;
				DummyDate=RxDate;
				if not missing(codesupply) then RxSup = CodeSupply;
				RxSupEndDt=DummyDate+RxSup-1; 	
				drop RxDate;
				output;
			end;
	        end;
	        stop;
		run;

		proc sort data=_temp_PreDrugs1 nodupkey;
			by _all_;
		run;

		%isdata(dataset=_temp_PreDrugsComb);
		%if %eval(&nobs.>0) %then %do;
	 	data _temp_PreDrugsComb1(rename=DummyDate=RxDate);
	        if 0 then set PtsList;
	        declare hash pt1 (hashexp:16, dataset:"PtsList");
	        pt1.definekey('PatId');
	        pt1.definedone();

	        do until(eof1);
	          set _temp_PreDrugsComb end=eof1;
	          if pt1.find()=0 then do;
			  	length DummyDate 4.;
				DummyDate=RxDate;
				if not missing(codesupply) then RxSup = CodeSupply;
				RxSupEndDt=DummyDate+RxSup-1; 	
				drop RxDate;
				output;
			end;
	        end;
	        stop;
		run;

		proc sort data=_temp_PreDrugsComb1 nodupkey;
			by _all_;
		run;
		%end;

		proc sort nodupkey data=_temp_PreDrugs(keep=StudyName StockGroup NDC Comorbcode CovarNum CovFrom CovTo CARESETTINGPRINCIPAL codedays excludesupply) out=PreDrugParams;
			by StudyName StockGroup;
		run;
		%ISDATA(dataset=_temp_PreDrugsComb);
		%if %eval(&nobs.>0) %then %do;
			proc sort nodupkey data=_temp_PreDrugsComb(keep=StudyName StockGroup NDC Comorbcode CovarNum CovFrom CovTo CARESETTINGPRINCIPAL codedays excludesupply) out=PreDrugParamsComb;
				by StudyName StockGroup;
			run;

			data PreDrugParams;
				set PreDrugParams PreDrugParamsComb;
			run;

			proc sort nodupkey data=PreDrugParams(keep=StudyName StockGroup NDC Comorbcode CovarNum CovFrom CovTo CARESETTINGPRINCIPAL codedays excludesupply);
				by StudyName StockGroup;
			run;
		%end;

		*separate out covariates;
		data dplocal.elixmeds (drop=covfrom covto)
			 _temp_PreMeds2;
			set _temp_PreMeds1;
				
			if Comorbcode eq 1 then output dplocal.elixmeds;
			else output _temp_PreMeds2;
		run;

		proc datasets nowarn noprint lib=work; 
			delete _temp_PreMeds1 _temp_PreMeds _temp_PreDrugs _temp_PreDrugsComb; 
		quit;

		*loop through enrollment datasets to shave and stockpile;
		%do enr=1 %to &numenrollgroup.;

		    %ms_shaveoutside(reffile=dplocal.enr_&enr.,
	                     refstart=Enr_Start,
	                     refend=Enr_End,
	                     KeepPartBf=N,
	                     ToShaveFile=_temp_PreMeds2,
	                     ToShaveStart=ADate,
	                     ToShaveEnd=ADate,
	                     outfile=dplocal.premeds_&enr.);

			%ms_shaveoutside(reffile=dplocal.enr_&enr.,
	                   refstart=Enr_Start,
	                   refend=Enr_End,
	                   KeepPartBf=N,
	                   ToShaveFile=_temp_PreDrugs1,
	                   ToShaveStart=RxDate,
	                   ToShaveEnd=RxSupEndDt,
	                   outfile=_temp_PreDrugs2);

			%ISDATA(dataset=_temp_PreDrugsComb1);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;
			%ms_shaveoutside(reffile=dplocal.enr_&enr.,
	                   refstart=Enr_Start,
	                   refend=Enr_End,
	                   KeepPartBf=N,
	                   ToShaveFile=_temp_PreDrugsComb1,
	                   ToShaveStart=RxDate,
	                   ToShaveEnd=RxSupEndDt,
	                   outfile=_temp_PreDrugsComb2_&enr.);
			%END;

			*Apply stockpiling - this can only be done once if no stockpiling file is specified
			*if there is a stockpiling file, this will be done in the group loop;
			%ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
			%IF %EVAL(&NOBS.=0) %THEN %DO;
				*set defaults;
				%let SAMEDAY=aa;
				%let SUPRANGE=0<-HIGH;
				%let AMTRANGE=0<-HIGH;
				%let PERCENTDAYS=;
			   
				%MS_STOCKPILING(INFILE=_temp_PreDrugs2,	  		
			                    CLMDATE=RxDate,                            		
			                    CLMSUP=RxSup,                              		
			                    CLMAMT=RxAmt,	                           		
			        		    PROCFLAG=,                                 		
			        		    PERCENTDAYS=&PERCENTDAYS.,                         		
			        		    GROUPING=StudyName StockGroup excludesupply,                   		
			        		    SAMEDAY=&SAMEDAY.,                                 	
			        			SUPRANGE=&SUPRANGE.,									
			        			AMTRANGE=&AMTRANGE.,
			        		    ID=,                                       		
			        		    OUTFILE=_temp_PreDrugs3,  		
			        			OUTFILEEXCL=_temp_PreDrugs3excl
			                    );

				*Reshave;
				%ms_shaveoutside(reffile=dplocal.enr_&enr.,
		                   refstart=Enr_Start,
		                   refend=Enr_End,
		                   KeepPartBf=N,
		                   ToShaveFile=_temp_PreDrugs3,
		                   ToShaveStart=RxDate,
		                   ToShaveEnd=ExpireDt,
		                   outfile=dplocal._temp_predrugs_&enr.);

			proc datasets lib=work noprint nowarn; 
				delete _temp_PreDrugs2 _temp_PreDrugs3 _temp_PreDrugs3excl;
			quit;

			proc sort data=dplocal._temp_predrugs_&enr.;
				by StudyName StockGroup;
			run;

			%end;
		%end;

		proc datasets lib=work noprint nowarn; 
			delete _temp_PreMeds2 _temp_PreDrugs1;
		quit;

		*Loop through groups to add covariate info;
		%do Group=1 %to &Numgroup.;

			data _null_;
				set &groupdefdata.;
				if _n_=&group. then do;
		        call symputx('ITGROUP',strip(&groupvar2.));
				%if "&groupvar" = "analysisgrp" %then %do;
				call symputx('primarygrp', strip(primary));
				%end;
				end;
			run;

			*Determine correct enrollment span for group;
			%if "&data" = "mstr_mi" %then %do;
				data _null_;
					set infolder.&micohortfile.;
						if cats(lowcase(milgrp), '_eoi') = lowcase("&itgroup.") then do;
							call symputx('groupname', groupname);
						end;
				run;
				data _null_;
					set cohortfile_for_enr;
					if upcase(cohortgrp)=upcase("&groupname.") then do;
						call symputx('ENRNUM',enrollmentnum);     
					end;
				run;
			%end;
			%else %do;
				data _null_;
					set cohortfile_for_enr;
					%if "&groupvar" = "group" %then %do;
					if upcase(cohortgrp)=upcase("&itgroup.") then do;
						call symputx('ENRNUM',enrollmentnum);     
					end;
					%end;
					%if "&groupvar" = "analysisgrp" %then %do;
					if upcase(cohortgrp)=upcase("&primarygrp") then do;
						call symputx('ENRNUM',enrollmentnum);     
					end;
					%end;
				run;
			%end;
			
			%IF %eval(&type.) = 2 %then %do;
				*Creating index reference file;
				proc sort data=&RUNID._&data. out= _ptsgroup(keep=patid &groupvar.) nodupkey;
						by patid &groupvar.;
						where &groupvar. in ("&ITGROUP.", "&ITGROUP._nvrexp");  
					run;

				proc sql noprint;
					select count(distinct &groupvar.) as &groupvar. into: Ngrp
					from _ptsgroup;
				quit;
			%end;

			*Stockpile and shave for each group according to stockpiling specifications;
			%ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
			%IF %EVAL(&NOBS.>0) %THEN %DO;
				*set defaults;
				%let SAMEDAY=aa;
				%let SUPRANGE=0<-HIGH;
				%let AMTRANGE=0<-HIGH;
				%let PERCENTDAYS=;

			
     			data &STOCKPILINGFILE.;
			        set infolder.&STOCKPILINGFILE.;
			        where group="&ITGROUP.";
			        call symputx("SAMEDAY",strip(SameDay));
			        call symputx("SUPRANGE",strip(SupRange));
			        call symputx("AMTRANGE",strip(AmtRange));
			        call symputx("PERCENTDAYS",put(PercentDays,best.));
			   run;


				%MS_STOCKPILING(INFILE=_temp_PreDrugs2,	  		
			                    CLMDATE=RxDate,                            		
			                    CLMSUP=RxSup,                              		
			                    CLMAMT=RxAmt,	                           		
			        		    PROCFLAG=,                                 		
			        		    PERCENTDAYS=&PERCENTDAYS.,                         		
			        		    GROUPING=StudyName StockGroup excludesupply,                   		
			        		    SAMEDAY=&SAMEDAY.,                                 	
			        			SUPRANGE=&SUPRANGE.,									
			        			AMTRANGE=&AMTRANGE.,
			        		    ID=,                                       		
			        		    OUTFILE=_temp_PreDrugs3,  		
			        			OUTFILEEXCL=_temp_PreDrugs3excl
			                    );

				*Reshave;
				%ms_shaveoutside(reffile=dplocal.enr_&enrnum.,
		                   refstart=Enr_Start,
		                   refend=Enr_End,
		                   KeepPartBf=N,
		                   ToShaveFile=_temp_PreDrugs3,
		                   ToShaveStart=RxDate,
		                   ToShaveEnd=ExpireDt,
		                   outfile=dplocal._temp_predrugs_&enrnum._&group.);


				%ISDATA(dataset=_temp_PreDrugsComb2_&enrnum.);
				%if %eval(&nobs.>0) %then %do;
				data dplocal._temp_predrugs_&enrnum._&group.;
					set dplocal._temp_predrugs_&enrnum._&group.
						_temp_PreDrugsComb2_&enrnum.(keep=patid studyname stockgroup rxdate rxsup rxamt RxSupEndDt enr_start enr_end excludesupply rename=RxSupEndDt=expiredt);
				run;
				%end;

				proc sort data=dplocal._temp_predrugs_&enrnum._&group.;
					by StudyName StockGroup;
				run;

			%end;
			%else %do;

				%ISDATA(dataset=_temp_PreDrugsComb2_&enrnum.);
				%if %eval(&nobs.>0) %then %do;
				data dplocal._temp_predrugs_&enrnum._&group.;
					set dplocal._temp_predrugs_&enrnum.
						_temp_PreDrugsComb2_&enrnum.(keep=patid studyname stockgroup rxdate rxsup rxamt RxSupEndDt enr_start enr_end excludesupply rename=RxSupEndDt=expiredt);
				run;
				%end;

				proc sort data=dplocal._temp_predrugs_&enrnum. out=dplocal._temp_predrugs_&enrnum._&group.;
					by StudyName StockGroup;
				run;

			%end;

			%if %eval(&type.) = 2 & &Ngrp. = 2 %then %do;
				data PreDrugs_&group.;
				 merge dplocal._temp_predrugs_&ENRNUM._&group.
					  PreDrugParams;
				 by StudyName StockGroup;
				 drop NumDispensing ExpireDt;
				run;

				proc sql noprint undo_policy=none;
					 create table PreDrugs_&group. as
					 select a.*, b.&groupvar.
					 from PreDrugs_&group. a inner join _ptsgroup b
					 on a.patid = b.patid;
				quit;

				proc sql noprint undo_policy=none;
					 create table PreMeds_&group. as
					 select a.*, b.&groupvar.
					 from dplocal.premeds_&group. a inner join _ptsgroup b
					 on a.patid = b.patid;
				quit;
			%end;
			%else %do;
				data PreDrugs_&group.;
					merge dplocal._temp_predrugs_&ENRNUM._&group.
					 	  PreDrugParams;
					by StudyName StockGroup;   
					format &groupvar. $40.;
					&groupvar. = "&ITGROUP.";   
					drop NumDispensing ExpireDt;
				run;

				data PreMeds_&group.;
					set dplocal.premeds_&ENRNUM.;
					format &groupvar. $40.;
					&groupvar. = "&ITGROUP.";
				run;
			%end;

			%if %eval(&group.) =1 %then %do;
				data PreDrugs; 
					set PreDrugs_&group.;
				run;
				data PreMeds; 
					set PreMeds_&group.;
				run;
			%end;
			%else %do;
				proc append base=PreDrugs data=PreDrugs_&group. force; run;
				proc append base=PreMeds data=PreMeds_&group. force; run;
			%end;

		    proc datasets library=work nolist nowarn;
		    	delete predrugs_: premeds_: ElixMeds_:;
		    quit;  
		    proc datasets library=dplocal nolist nowarn;
		    	delete _temp_predrugs_&enrnum._&group.;
		    quit;  
		%end;
		proc datasets library=work nolist nowarn;
			delete PreDrugParams _temp: _temp_PreDrugs2 _temp_PreDrugs3 _temp_PreDrugs3excl;
		quit;  
		proc datasets library=dplocal nolist nowarn;
			delete premeds_: _temp_predrugs_:;
		quit;  
	%END;
    %mend;
    %RunCovExtraction;

	/******************************************/
	/* Charlson Comorbidity/Elixhauser Index) */
	/******************************************/

    %macro RunCciElix;

    %ISDATA(dataset=Comorbfile);	
    %IF %EVAL(&NOBS.>0) %THEN %DO;


		*Break down Meds in procs and diags;
        data dplocal._cci_elix_dx
             dplocal._cci_elix_px;
            set dplocal.elixmeds;
            *where Comorbcode eq 1;
            if upcase(codecat)="DX" then output dplocal._cci_elix_dx;
            if upcase(codecat)="PX" then output dplocal._cci_elix_px;
        run;

        %do Group=1 %to &Numgroup.;

			*Select group;
			data _null_;
				set &groupdefdata.;
				if _n_=&group. then do;
		           call symputx('ITGROUP',strip(&groupvar2.));
				end;
			run;

		 	*Process input file;
            data _null_;
               set Comorbfile;
               if group = "&itgroup." then do;
                    call symputx('ITGROUP',Group);
                    call symputx('comorbfrom',coalesce(comorbfrom,-999999)); 
                    call symputx('comorbto',coalesce(comorbto,999999));
                    call symputx('ccigroup',ccigroup);
                end;
            run;
            %put &ITGROUP. &comorbfrom. &comorbto. &ccigroup.;

            *The Charlson/Elixhauser combined comorbidity score will be calculated;
            data _IndexFile;
            set &RUNID._&data.;
            where &groupvar.="&ITGROUP.";
            Keep Patid indexdt;
            run;

            %ms_cci_elix(COMORBFROM=&comorbfrom.,
                         COMORBTO=&comorbto.,
                         INFILE=_IndexFile, 
                         PATID=PatId,
                         INDEXDT=IndexDt, 
                         PREOUTFILE=_PreOutFile,
                         DIAGFILE=dplocal._cci_elix_dx,
                         DX_DT_VAR=Adate,
                         DX_CODETYP=Codetype,
                         DX_CODE=Code,
                         PROCFILE=dplocal._cci_elix_px, 
                         PX_DT_VAR=Adate,
                         PX_CODETYP=Codetype,
                         PX_CODE=Code,
                         CCI_LOOKUP=infolder.Comorbcodes, 
                         OUTFILE=_ITcci_elix,
                         CCIGROUP=&ccigroup.,
                         KEEPALLDUM=0);   /*&keepalldum.*/

        	*Post Process;
            data _ITcci_elix;
	            set _ITcci_elix;
	            format &groupvar. $40.;
	            &groupvar.="&ITGROUP."; 
	            length ccielixgrp $10.;
	            format ccielixgrp $10.;
	            ccielixgrp=COMBINED_SCORE;
            run;

            *Combine Util for multiple groups;
            %IF %EVAL(&group.=1) %THEN %DO;
                data _ALLcci_elix;
                set _ITcci_elix;
                run;
            %END;
            %ELSE %DO;
                proc append base=_ALLcci_elix
                            data=_ITcci_elix force;
                run;
            %END;

            *Clean-up;
            proc datasets library=work nolist nowarn;
            	delete _ITcci_elix _IndexFile _PreOutFile;
            quit;

        %END; *group loop;

        proc sort data=_ALLcci_elix;
        by &groupvar. PatId IndexDt;
        run;

		%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;

		    data _NewPatients;
		    set DPLocal.&RUNID._&data.;
		    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
		    keep &groupvar. PatId IndexDt;
		    run;

		    proc sort nodupkey;
		    by &groupvar. PatId IndexDt;
		    run;

		    data _ALLcci_elix;
		    merge _ALLcci_elix(in=a)
		          _NewPatients(in=b);
		    by &groupvar. PatId IndexDt;
		    if a and b;
		    run;
		%end;

        *Update Master file with utilization data;
        data DPLocal.&RUNID._&data.;
        merge DPLocal.&RUNID._&data.
              _ALLcci_elix;
        by &groupvar. PatId IndexDt;
        run;

        proc datasets library=work nolist nowarn;
        delete _BaseEnc IndexFile Elix_infile_sort Dsout Cci_diag_temp Cci_diag_lookup;
        quit;

    %end;
	%else %do;		
		data DPLocal.&RUNID._&data.;
		set DPLocal.&RUNID._&data.;
		COMBINED_SCORE="";
		COMBINED_SCORE_NUM=.;
		ccielixgrp="";
		run;
	%end;

	proc datasets nowarn nolist noprint lib=dplocal;
		delete _cci_: elixmeds;
	quit;
    %mend;

    %RunCciElix;

	/****************/
	/* Prepare ADS	*/
	/****************/

    %macro ADS;

		%let t1_table1vars = ;
		%let t2_table1vars = ;
		%let t3_table1vars = ;
		%let t4_table1vars = ;
		%let t5_table1vars = ;
		%let analysiscohort = ;
		%let mi_vars = ;
		%let tree_vars = ;

		%if %eval(&type.=2) %then %do;
			%if %eval(&multievent.=0)& %eval(&overlap.)=0 & %eval(&concomitant.=0) %then %do;
				%let t2_table1vars = NumEvents tte blackoutper EpisodeType LastLookFollowed: EpisodeEndDt t2atriskstart ttc;
			%end;
			%else %do;
				%let t2_table1vars = NumEvents tte blackoutper EpisodeType LastLookFollowed: EpisodeEndDt;
			%end; 
		%end;
		%if %eval(&type.=3) %then %do;
			%let t3_table1vars = DAYS_FROM: analysiscohort censor:;
			%let analysiscohort = analysiscohort censor:;
		%end;
		%if %eval(&type.=4) %then %do;
			%let t4_table1vars = delnum prepostind ga_birth ;
			%if "&data" = "mstr_mi" %then %do;
				%let mi_vars = LastLookFollowed ga_first exp_pre exp_t1 exp_t2 exp_t3 adjusteddisp_pre adjusteddisp_t1 adjusteddisp_t2 adjusteddisp_t3 birth_enroll enroll_diff 
							   matchmethod birth_type linkedid; 
				%if "&analysis" = "tree" %then %do; 
					%let tree_vars = ttc followuptime outcomestart ;
				%end;
			%end;
		%end;
		%if %eval(&type.=5) %then %do;
			%let t5_table1vars = episodenum;
		%end;
	
        *Creating the base file to extract data 
         and then divide into different PeriodId;
        data _Base;
        set DPLocal.&RUNID._&data.;
			if _n_ = 1 then do;
				dsid = open("DPLocal.&RUNID._&data.");
				if varnum(dsid,"FEventDt") = 0 then FEventDt =.;
				if varnum(dsid,"NumEvents") = 0 then NumEvents =0;
				if varnum(dsid,"blackoutper") = 0 then blackoutper =0;
				rc= close(dsid);
			end;
			drop rc dsid;

        	rename AgeGroup=Age_Cat
	               FEventDt=EventDt
	               COMBINED_SCORE_NUM=COMORBIDSCORE;

        	keep PatID IndexDt Year &groupvar. Age AgeGroup AgeGroupNum Sex Race hispanic FEventDt IndexLook IndexDtLookEndDt
             	 COMBINED_SCORE_NUM NumIP NumED NumAV NumOA NumIS cb_reg hhs_reg hispanic month race sex state year zip3 zip_uncertain 
              	 NumGeneric NumClass NumRx &t2_table1vars. &t3_table1vars. &t4_table1vars. &t5_table1vars. &mi_vars. &tree_vars. &lookdt.;
        run;

		proc datasets lib=work nolist;
       		modify _Base;
	        attrib _all_ label=''; *Remove labels;
      	quit;

        proc sort data=_base;
        	by &groupvar. Patid IndexDt %if %eval(&type.=5) %then %do; episodenum %end;;
        run;

		%if %eval(&type.=5) %then %do;
		data _base;
			set _base;
			by &groupvar. patid indexdt episodenum;
			if first.patid;
		run;
		%end;

        /**************************/
        /* Creating Covar1-CovarN */
        /**************************/

        *Combine records;
        data _CombineRecords;
	        length Adate ExpireDt 4.;  * to avoid w a r n i n g s;
	        set PreDrugs(rename=RxDate=ADate)
	            PreMeds;
	        format Adate ExpireDt date9.;
	        ExpireDt=Adate+max(1,RxSup)-1;
	        where studyname ne '';
	        keep &groupvar. PatId Adate ExpireDt studyname CovarNum covfrom covto codedays excludesupply;
        run; 

        proc datasets library=work nowarn nolist;
        delete _Drugs CovMeds PreMeds;
        quit;

	    /*Finding those claims in covariatewindow*/
        proc sql noprint;
        create table _Covar(where=((ExcludeSupply='N' and WithSupply=1) | (ExcludeSupply='Y' and WithoutSupply=1))) as
        select distinct pts.Patid,
                        pts.&groupvar.,
                        pts.IndexDt,
                        Rec.studyname,
						Rec.Adate,
						Rec.covfrom,
						Rec.covto,
                        Rec.CovarNum,
						Rec.codedays,
						Rec.ExcludeSupply,
						%MS_PeriodsOverlap(period1=coalesce(pts.IndexDt+covFrom,-99999) COALESCE(pts.indexdt+CovTo/*-1*/,99999),                                             
                                			period2=Rec.ADate Rec.ExpireDt)as WithSupply ,
				   		%MS_PeriodsOverlap(period1=coalesce(pts.IndexDt+covFrom,-99999) COALESCE(pts.indexdt+CovTo/*-1*/,99999),                                             
                                			period2=Rec.ADate)as WithoutSupply
        from _base as pts, 
             _CombineRecords as Rec
        where pts.Patid = rec.Patid and pts.&groupvar. = rec.&groupvar.
        order by pts.&groupvar.,
                 pts.Patid,
                 pts.IndexDt;
        quit;

		/*Count number of ADate per covariate and only keep covariate when freq >=codedays*/
		proc means data=_Covar noprint nway missing;
			class &groupvar. patid indexdt studyname covfrom covto covarnum codedays / missing;
			output out=_Covar(drop=_: where=(freq>=codedays)) n=freq;
		run;

        *Squaring;
        data _Covar;
        set  studynames
             _Covar(drop=freq codedays);
        Rec=1;
        run;

        *One record per Patid &groupvar. IndexDt;
        proc transpose data=_Covar out=_Covar(drop=_NAME_) prefix=COVAR;
        var Rec;
        by &groupvar. Patid IndexDt;
        id Covarnum;
        idlabel studyname;
        run;

		/******************************/
		/*Verify if any combcovariates*/
		/******************************/		
	    %ISDATA(dataset=infolder.&COVARIATECODES.);	
		%IF %EVAL(&NOBS.>0) %THEN %DO;
			data _COMBO;
			set infolder.&COVARIATECODES.;
			where codecat in ("CC");
			run;

			proc sort data=DPLocal.&RUNID._&data.;
				by &groupvar. PatId IndexDt;
			run;

			%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
			    data _NewPatients;
			    set DPLocal.&RUNID._&data.;
			    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
			    keep &groupvar. PatId IndexDt;
			    run;

			    proc sort data=_NewPatients nodupkey;
			    by &groupvar. PatId IndexDt;
			    run;
			%end;

			%ISDATA(dataset=_COMBO);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;				
				data _COMBO;
				set _COMBO;
				format RuleLong $1000.;
				RuleLong=code;
				%do j=1 %to &numcovars.;
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),(" &j. ")," COVAR&j. "));
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),("(&j. "),"(COVAR&j. "));
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),(" &j.)")," COVAR&j.)"));
				%end;
				run;

				*Create CombCovar variables;
				%do ii=(&numcovars.+1) %to (&numcovars.+&numcomb.);
					data _null_;
					set _COMBO (where=(Covarnum=&ii.));
					call symput ('Rule',RuleLong);
					call symput ('Label',Studyname);
					run;
					%put &Rule.;

					data _covar;
					set _covar;
					COVAR&ii.=0;
					if &rule. then do;
					COVAR&ii.=1;
					end;
					label COVAR&ii.="&Label.";
					run;

				%end;*do loop numcomb;

				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
				    
					proc sort data=_Covar;
				    by &groupvar. PatId IndexDt;
				    run;

				    data _Covar;
				    merge _Covar(in=a)
				          _NewPatients(in=b);
				    by &groupvar. PatId IndexDt;
				    if a and b;
				    run;

				%end;

		        *Add dummies to master file (and will also order the vars by covarN);
		        %macro PutBack;
		        data _base;
		        merge _base(in=a)
		              _Covar(keep=&groupvar. PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;
													   %do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k. )%end;);
		        by &groupvar. PatId IndexDt;
		        if a;  
		        run;

		        data DPLocal.&RUNID._&data.;
		        merge DPLocal.&RUNID._&data.(in=a)
		              _Covar(keep=&groupvar. PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;
													   %do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k. )%end;);
		        by &groupvar. PatId IndexDt;
		        if a;  

				    *fill covars with 0s;
		            %do j=1 %to &numcovars.;%str(covar&j.=covar&j.>0;)%end;
					%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k.=covar&k.>0; )%end;
		        run;
		        %mend;
		        %PutBack;
			%END;
			%ELSE %DO;
				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;				    

				    data _Covar;
				    merge _Covar(in=a)
				          _NewPatients(in=b);
				    by &groupvar. PatId IndexDt;
				    if a and b;
				    run;

				%end;

		        *Add dummies to master file (and will also order the vars by covarN);
		        %macro PutBack;
		        data _base;
		        merge _base(in=a)
		              _Covar(keep=&groupvar. PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;);
		        by &groupvar. PatId IndexDt;
		        if a;  
		        run;

		        data DPLocal.&RUNID._&data.;
		        merge DPLocal.&RUNID._&data.(in=a)
		              _Covar(keep=&groupvar. PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;);
		        by &groupvar. PatId IndexDt;
		        if a;  

				    *fill covars with 0s;
		            %do j=1 %to &numcovars.;%str(covar&j.=covar&j.>0;)%end;
		        run;
		        %mend;
		        %PutBack;
			%END;
		%END;

		%let delnum = ;
		/*Need to include DelNum for Type 4 non-pregnant group because it may not be unique on group/patid/indexdt*/
		%if "&data" = "mstr_nopreg" %then %do; 
			%let delnum = delnum; 
		%end;

        %macro forAdsloop;
        %do i=&PERIODIDSTART. %to &PERIODIDEND.;
         
			proc sort data =_base;
				by &groupvar. PatId IndexDt &delnum.;
			run;

            data DPLocal.&RUNID._ads_&data._&i.;
            set _Base;

            if missing(NumGeneric) then NumGeneric=0;
            if missing(NumClass) then NumClass=0;
            if missing(NumRx) then NumRx=0;

			*if Type = 2 then: 
				1. Adjust tte for episodes going beyond the PERIODID
				2. Add followuptime and event vars ;

			*if Type = 4 and MI then:
				1. Add event var ;

			%if %eval(&type.=2) %then %do;
	            if IndexDt <= IndexDtLookEndDt and EpisodeEndDt > min(LastLookFollowedDt,IndexDtLookEndDt) then do;
	               if EventDt > min(LastLookFollowedDt,IndexDtLookEndDt) then EventDt = .; 
	               tte = min(LastLookFollowedDt,EventDt,IndexDtLookEndDt) - IndexDt +1 -  BlackOutPer;
	            end;

				*Variables needded for the ADS;
	            Followuptime=.;
	            Event=0;
	            Followuptime=tte;
	            Event=EventDt>0;
	    
				drop BlackOutPer EpisodeEndDt;
			%end;
			%if %eval(&type.=4) & "&data" = "mstr_mi" %then %do;
	            Event=0;
	            Event=EventDt>0;
			%end;

            *fill covars with 0s;
            %do j=1 %to &numcovars.;%str(covar&j.=covar&j.>0;)%end;
			%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k.=covar&k.>0; )%end;

            *Output incremental datasets;
            if &lookdt. <= IndexDtLookEndDt and indexlook <= &i. then output DPLocal.&RUNID._ads_&data._&i.;

            *Cleanup to keep only required in specs;
            drop IndexDtLookEndDt;

			rename IndexLook=time;
            run;

			*Carry forward information from previous look in ads_mstr;
			%if (%eval(&i.>1) & %eval(&type.=2)) & "&analysis" ne "tree" %then %do;
				%let previous = %eval(&i-1); 
				data DPLocal.&RUNID._ads_&data._&i.;
				merge DPLocal.&RUNID._ads_&data._&i.
              	  	  &DPLOCALPOINTER..&RUNID._ads_&data._&previous. (where=(Time < &PERIODIDSTART. and LastLookFollowed > 0)
															  keep=Time LastLookFollowed &groupvar. PatId IndexDt Followuptime Event
															  %do j=1 %to &numcovars.;%str(covar&j. )%end;
															  %do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k. )%end;);
	        	by &groupvar. PatId IndexDt;
	        	run;
			%end;
		%end;
		%mend forAdsloop;
        %forAdsloop;

        /****************************/
		/** Compute baseline table **/
        /****************************/

		%if %str("&SURVEILLANCEMODE") = %str("") %then %do;
		%ms_createdistbaselinetable(where=1, t3out=);
		%end;

		%if %eval(&type.=3) & "&analysis" ne "tree" %then %do;
      		%ms_createdistbaselinetable(where=analysiscohort=1, t3out=_an);
		%end;

    %mend;
    %ADS;

	%end;
	%else %do;
	%put WARNING: No users were identified in DPLOCAL.&RUNID._&data.. A baseline table will not be produced for this cohort.;
	%end;

	/*output runtime*/
	%ms_stoptimer(timestart=cidacovruntime);
	%ms_outputruntimes(timevar=cidacovruntime, step=%str(Covariate/Utilization (cidacov)), group=%str(data = &data.), monitoringperiod=);
	 
	%put NOTE: ******** END OF MACRO: ms_cidacov v3.5 ********;

%mend ms_cidacov;
