****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_covariate_adjustment.sas  
*
* Created (mm/dd/yyyy): 06/11/2014
* Last modified: 06/30/2019
* Version: 4.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  Analytic code to compute propensity score and perform nearest neighbor matching wihtout replacement. 
*  Additionally analytic capabilities include multiple factor matching, computation of HDPS variables, 
*  and compute risk sets
*  
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*   1.1       09/10/14   MSOC(JRP)  Added proc options to gather Java-related environment data.
*                                   Added defensive coding to strip SAS dataset extension from COMPARISON.
*                                   Added VARIABLE parameter to allow users to run EITHER 1:1 OR 
*                                      variable (up to 100) matching rather than both.
*
*   1.2       12/01/14   SL(EM)     Changed variable matching ratio from 100 to 10.
*                                   Included YEAR and TIME to logit model.
*                                   Included  &covarlist to class statement of the HDPS=N logit.
*
*   1.3       02/19/2015 SL(EM)     Changed stratamatch such that matching is always executed
*                                   Added bypass code in stratamatch for USE_HDPS=N.
*                                   Modified initHDPS and local proc logistic to keep all convergence 
*                                      details and store them into msoc.&RUNID._estimates_&comp._&look.
*                                   Output varinfo to msoc.&RUNID._VARINFO_&comp._&LOOK.
*                                   Only retain PS estimate used to match on all output 
*                                      datasets
*
*   2.0       03/06/15    SL(EM)    Major changes to include the creation of aggregated datasets (riskset, 
*                                      Riskdiff and survival)in addition to the matched patient level data which
*                                      is now optional
*                                   Subgroup analysis re-matching is now handled at the distributed level
*                                      and defined using the ANALYTICSUBGROUPS input file (and macro parameter).  
*                                      Use the INDLEVEL=Y macro parameter to output the patient level data to MSOC
*                                      in addition to the aggregated files.
*
*   2.1       08/26/15    SL(EM)    Restriction to M and F removed
*                                   Corrected matchtables macro call
*
*   2.2       11/09/15    SL(EM)    Removed TEMPLATETABLE macro variable because Excel dependency is removed
*                                   from matchtable
*
*   2.3       12/23/15    SL(EM)    Made MatchedinFull optional.
*
*   2.4       01/15/16    SL(EM)    Remove Java. 
*									Resurrect keep statement.
*									Move HDPS variable from TYPE2FILE to COMPARISON input file.
*									Remove MODEL variable from COMPARISON file.
*
*	2.5		  08/10/16	 AP(SOC)	Added 2 new HDPS dimension files (DX10 and PX10) for ICD10 (QCI-186)
*									Added back in requester selected HDPS parameters (QCI-192)
*
*	2.6		  08/15/16	  DM		Added Surveillance related code
*
*	2.7		  10/17/16	  DM		Fixed issues related to surveillance
*
*   2.8		  05/05/17	  AP		Added race and hispanic as potential subgroup variables
*
*	2.9	 	  10/05/17	  AP		-Added MATCH_STRAT_KEEP0 parameter to handle episodes with 0 days follow-up
*									 due to blackout period (QRP-319)
*									-Recoded A/U sex values (QRP-453)
*
*	2.10	  11/20/17	  AP	    -Removed MATCH_STRAT_KEEP0 parameter (QRP-466)
*
*	2.11	  03/14/18	  RR	    Ability to run Multiple Factor Matching algorithm (QRP-322)
*
*	2.12	  06/06/18	  AP	    Compatibility modifications with QRP 6.0.0
*
*	3.0		  10/20/18	  AP	    Added Type 4 PS analysis for QRP 7.0.0
*									Simplified variable naming (event/followup and matchID vars)
*
*	3.1		  02/12/19	  AP		Updated requestID and common components parameters to remove MS (i.e. MSReqID --> ReqID)
*
*	4.0		  06/30/19	  AP		- Added Tree Extraction for signal detection analysis
*									- Incoporated as macro call within ms_cidanum
*									- Replaced comp_order with analysisgrp
*									- Merge MFMFILE and COMPARISON files
*								    - Added match attrition table
*									- Always include race/hispanic in Table 1
*
****************************************************************************************************;

%macro ms_covariateadjustment();

	%put =====> MACRO CALLED: ms_covariateadjustment v4.0;

	/*********************************************************************
	  SET UP 
	**********************************************************************/

	/* The following option is required to prevent the following warn in the log:
  	   The quoted string currently being processed has become more than 262 characters long.  
       You might have unbalanced quotation marks */

	options noquotelenmax;

	%let ORIGINDLEVEL=%LOWCASE(&INDLEVEL.);

	%let match_Error=;
	%let AnalysisInput = ;

	/*Utility attrition macro*/
    %macro matchingattrition(file=,ToExcl=);

	    proc sql noprint;
		    select count(distinct PatId) into: NumToExcl1
			from &file.
			where exposure = 1 and not (&ToExcl.);

		    select count(distinct PatId) into: NumToExcl0
			from &file.
			where exposure = 0 and not (&ToExcl.);

		quit;
		%put &NumToExcl1. &NumToExcl0.;

		data &file.;
		    set &file.;
		    where not (&ToExcl.);
	    run;

	    %ISDATA(dataset=dplocal._matchattrition);
	    %IF %EVAL(&NOBS.>=1) %THEN %DO;
	        data dplocal._matchattrition;
	        set dplocal._matchattrition end=eof;
	        output;
	        if eof then do;
	            Level=input("&level.",best.);
		        eoi_remaining=input("&NumToExcl1.",best10.);
		        ref_remaining=input("&NumToExcl0.",best10.);
	            call symputx("level",&level.+1);
	            output;
	        end;
	        run;
	    %END;
	    %ELSE %DO;
	        data dplocal._matchattrition;
	        format level best. eoi_remaining ref_remaining comma10.;
	        Level=input("&level.",best.);
	        eoi_remaining=input("&NumToExcl1.",best10.);
	        ref_remaining=input("&NumToExcl0.",best10.);
	        call symputx("level",&level.+1);
	        output;
	        run;
	    %END;
	
    %mend; 


	*For Type 2 - initialize followuptime. Type 4 uses logistic regression without followup time;
	%let followuptime = followuptime;
	%let t4psavars = ;
	%if %eval(&type.=4) %then %do;
		%if "&analysis" ne "tree" %then %do;
			%let followuptime = ;
		%end;
	%let t4psavars = prepostind;
	%end;

	*Determine Input data;
	%if %eval(&type.=2) %then %do;
		%let indata = dplocal.&runid._ads_mstr;
	%end;
	%if %eval(&type.=4) %then %do;
		%let indata = dplocal.&runid._ads_mstr_mi;
	%end;

	/*********************************************************************
	  LOOP THROUGH EACH PERIOD
	**********************************************************************/
	%do look = &PERIODIDSTART. %to &PERIODIDEND. ;  ** Loop through looks ;

		%if %sysfunc(exist(&INDATA._&LOOK.)) = 0 %then %do;	
	  		%put WARNING: No users in the look &LOOK.;
			%goto nextlook;
	  	%end;	

	  	%put ======> Look = &look.;

		%global prior;
		%let prior = %eval(&look-1); 

		%let DPLOCALPOINTER=DPLocal;
		%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %let DPLocalPointer=DPLPrior;
		%put NOTE: DPLOCAL for prior run is set to &DPLocalPointer.;

		/*****************************************************************************
	  		CREATE MACRO VARIABLES FOR EACH COMPARISON.  LOOP THROUGH EACH COMPARISON.
		******************************************************************************/
	  	proc sql noprint;
		    select count(*) into :compcnt
		    from infolder.&comparison.;
	  	quit ;

	  	%do comp = 1 %to &compcnt.;  ** Loop through comparisons ;

			/*********************************************************************
			    START LOG
			**********************************************************************/

			/********************************************************
			   GET ADJUSTMENT START TIME
			 *******************************************************/

			%ms_starttimer(timestart=covadjustruntime);

			/*********************************************************************
			  KEEP DRUGS OF INTEREST - DEFINE EXPOSURE.  KEEP ONLY FIRST EXPOSURE
			  PER PATIENT.  DEFINE MACRO VARS. IF PATIENT HAS  SAME INDEX DATE 
			  FOR BOTH EXPOSURES THEN EXCLUDE THE PATIENT ENTIRELY.
			**********************************************************************/

			data _null_ ;
		    	set infolder.&comparison. ;
		      	if _n_ = %eval(&COMP.) then do ;

					call symputx('analysisgrp', analysisgrp);
			        call symputx('EOIGRP', eoi) ;
			        call symputx('REFGRP', ref) ;
					call symputx('RATIO',put(ratio, best.)) ;
					call symputx('matchanalysis', upcase(matchanalysis));

		 			*Do not ouput individual level data for never exposed group*;
			    	if index(lowcase(ref), "_nvrexp") > 0 then call symputx("INDLEVEL","n");
			    	else call symputx("INDLEVEL","%lowcase(&ORIGINDLEVEL.)");

					if upcase(matchanalysis) = "PS" then do;
		       	 		call symputx('CALIPER', put(CALIPER,best.)) ;
						score_type_3= 0;
				        score_type_4= 0;
				        score_type_5= 0;

						if UPCASE(HDPS)="N" then do;
				            call symputx('use_hdps',"N");
				            score_type_3= 1;
				        end;
				        if UPCASE(HDPS)="Y" and (CLASS="" AND NOCLASS="") then do;
				            call symputx('use_hdps',"Y");
				            score_type_5= 1;
				        end;
				        if UPCASE(HDPS)="Y" and not (CLASS="" AND NOCLASS="") then do;
				            call symputx('use_hdps',"Y");
				            score_type_4= 1;
				        end;

				        call symputx('psvar', "PSCORE");

				       	call symputx('score_type_3', put(score_type_3,best.)) ; 
				       	call symputx('score_type_4', put(score_type_4,best.)) ;
				       	call symputx('score_type_5', put(score_type_5,best.)) ;

				       call symputx('ClassVars', lowcase(class)) ;
				       call symputx('NoClassVars', lowcase(NoClass)) ;

				       call symputx('HDPSWinFrom', put(HDPSWinFrom,best.)) ;
				       call symputx('HDPSWinTo', put(HDPSWinTo,best.)) ;

					   call symputx('percentile',"percentile");
					end;

					if upcase(matchanalysis) = "MS" then do;
						/*Make Caliper 0*/
						call symputx('CALIPER',0);
				        call symputx('psvar', "MFMS");

						call symputx('matchvars', upcase(MATCHVARS));
			    		call symputx('percentile',"");
						call symputx('use_hdps','N');
					end;
				end;
			run;

		    %put =====>Now looping on comparison &comp.: &analysisgrp., matching on &psvar., ratio = &ratio.;
			%put =====>HDPS = &use_hdps; 

		    data temp3 ;
		    	set &indata._&look. ;

				format analysisgrp $40.;
				analysisgrp = "&analysisgrp.";
		    
			    if (UPCASE(Group) eq %UPCASE("&EOIGRP.")) then do ;
			    	exposure = 1 ;
			    	output ;
			    end ;
			    else if (UPCASE(Group) eq %UPCASE("&REFGRP.")) then do ;
			        exposure = 0 ;
			    	output ;
			    end ;
			    rename group = studyclass ;
		    run ;

		    proc sort data = temp3 ;
		    	by patid IndexDt ;
		    run ;

			data temp3;
		      set temp3 ;
		      by PatID IndexDt ;
			  	first_exposure = 0;
				nosameday = 0;
		     	if first.PatID then first_exposure = 1;
				if (first.IndexDt AND last.IndexDt) then nosameday=1;
					*recode same say for first exposure;
					if nosameday = 0 then first_exposure = 0;
				all=1;
				keepforlook = 0;
		    run ;

			/************************/
			/* Attrition levels 1-3 */
			/************************/

			 %let level = 1;

			 /*1*/	%matchingattrition(file=temp3,ToExcl=all=0);
		     /*2*/	%matchingattrition(file=temp3,ToExcl=nosameday=0);
		     /*3*/	%matchingattrition(file=temp3,ToExcl=first_exposure=0);


			/****************************************************************
			MAKE SURE THERE ARE NEW USERS
			*****************************************************************/
		    %let numNewExpgroups = 0;
		    %let numNewPatients = 0;
		  
		    proc sql noprint;
		      select count(distinct studyclass) into :numNewExpgroups
		      from temp3
		      where Time eq &look. ;

		      select count(distinct patid) into :numNewPatients
		      from temp3
		      where Time eq &look. ;
		    quit;
		    

			/*For tree extraction - need to save cumulative file for PS computation in later looks*/

			%if "&analysis"="tree" %then %do;

				/*look 1*/;
				%if %eval(&look.=1) %then %do;
					%if %eval(&comp.)=1 | %sysfunc(exist(dplocal.&runid._cumulative_mstr_for_ps))=0 %then %do;
						data dplocal.&runid._cumulative_mstr_for_ps;
							set temp3;
						run;
					%end;
					%else %do;
						data dplocal.&runid._cumulative_mstr_for_ps;
							set dplocal.&runid._cumulative_mstr_for_ps 
								temp3;
						run;
					%end;
				%end;
				%else %do;

				/*looks 2+*/
					%if %eval(&comp.)=1 | %sysfunc(exist(dplocal.&runid._cumulative_mstr_for_ps))=0 %then %do;
						data dplocal.&runid._cumulative_mstr_for_ps;
							set dplprior.&runid._cumulative_mstr_for_ps
							temp3; 
						run;
					%end;
					%else %do;
						data dplocal.&runid._cumulative_mstr_for_ps;
							set dplocal.&runid._cumulative_mstr_for_ps 
								temp3;
						run;
					%end;

				
					*Temp3 now contains all patients in all loosk;
					data temp3;
						set dplocal.&runid._cumulative_mstr_for_ps;
						where analysisgrp = "&analysisgrp";
					run;

					*Remove patients who were identified in a prior look in the other group;
					proc sort data = temp3 ;
			    		by patid IndexDt ;
			    	run;

					data temp3
						 temp3excl;
						set temp3;
						by patid indexdt;
						if first.patid then output temp3;
						else output temp3excl;	
						*because looks are non-overlapping, there will not be same day exposures in multiple groups;
					run;
		
					*Attrition level 4;
					*Because we do not want to remove prior look patients from temp3, will not use attrition macro;
					proc sql noprint;
						select count(distinct PatId) into: NumToExcl1
						from temp3
						where exposure = 1 and time =%eval(&look.);

					    select count(distinct PatId) into: NumToExcl0
						from temp3 
						where exposure = 0 and time =%eval(&look.);
					quit;
					%put &NumToExcl1. &NumToExcl0.;

					*Add to attrition dataset;
			        data dplocal._matchattrition;
			        	set dplocal._matchattrition end=eof;
			        output;
			        if eof then do;
			            Level=input("&level.",best.);
				        eoi_remaining=input("&NumToExcl1.",best10.);
				        ref_remaining=input("&NumToExcl0.",best10.);
			            call symputx("level",&level.+1);
			            output;
			        end;
			        run;
	 
				%end;
			%end;

			%if "&analysis" ne "tree" | %eval(&look.=1) %then %do;
			/*4*/ %matchingattrition(file=temp3,ToExcl=keepforlook=.);
			%end;

		    /* If no new users identified, abort program */
		    %if &numNewPatients = 0 %then %do;
		    	%put WARNING: No new users within calendar time window and age boundaries of this scenario have been identified;
		      	%goto continue ;
		    %end;

			/****************************************************************
				MACRO LIST PREDEFINED COVARIATE CONDITIONS 
			*****************************************************************/
  
		    %macro confounderlist();
		    	%if %sysfunc(exist(infolder.&COVARIATECODES.)) %then %do;
		    
		    	%global numc nocovarcondition keepcovars ;
		    
		    	%let nocovarcondition = 0;
		    
		    	proc sql noprint;
		    	  create table count AS
		    	  select DISTINCT UPCASE(studyname) as distinct_cov
		    	  from infolder.&COVARIATECODES.
		          order by distinct_cov asc;
		   	 
				  select COUNT(DISTINCT distinct_cov)
		    	  into :numc
		    	  from count;  

				  select max(keep)
		    	  into :has_keep
		    	  from infolder.&COVARIATECODES. ;
		        quit ;

		     	%if (&has_keep. ne 0) %then %do ;
			        proc sql noprint;           
			          create table covar_labels AS
			          select UPCASE(name) as varname, UPCASE(label) as label
			          from dictionary.columns
			          where libname eq 'WORK' and memname eq 'TEMP3' and UPCASE(name) like 'COVAR%'
			          order by label ;

			          create table keep_covars AS
			          select DISTINCT UPCASE(studyname) as label
			          from infolder.&COVARIATECODES.
			          where (keep eq 1)
			          order by label asc;
			                        
			          select count(*)
			          into :numkeep
			          from keep_covars ; 

			          create table keep_covar_vars AS
			          select varname
			          from keep_covars a inner join covar_labels b on (a.label = b.label) 
			          order by varname asc ;

			          select varname
			          into :COVARK1-:COVARK%trim(&numkeep.)
			          from keep_covar_vars
			          order by varname asc ;
			        quit ;
				%end ;
		    	%else %let numkeep = 0 ;
		    
		        %let keepcovars = ;
		        %do i = 1 %to &numkeep. ;
		        	%let keepcovars = &keepcovars &&covark&i. ;
		        %end ;
		    
		      	%end ;
		      	%else %do ;
			      	%global nocovarcondition keepcovars;
			      	/** IF NO COVARIATE CONDITIONS OR NO COVARIATE RX OR NO COVARIATE PROCEDURES DEFINED THEN DO **/
			      	%let keepcovars = ;
			      	%let nocovarcondition = 1;
		      %end;
		    %mend confounderlist ;
  		    %confounderlist();

			/*******************************************************************
				CREATE CLAIMS FILES FOR HDPS FOR ONLY PATS IN CURRENT COMPARISON
			********************************************************************/
    		%macro get_codes ;
   
				%let dimensionlist = ;

				%let FILECNT = %SYSFUNC(COUNTW(&HDVARSEL_INPUT.));
				%do i = 1 %to &FILECNT. ; 
					%global CODELIST&i. ;
					%let CODELIST&i. = %scan(&HDVARSEL_INPUT., &i.) ;

					/*Depending on query dates, some dimension files could be empty, 
					need to rebuild HDVARSEL_INPUT with only dimension files that have records*/

					%ISDATA(dataset=DPLocal.&&CODELIST&i..);	
					%IF %EVAL(&NOBS.>0) %THEN %DO;
						%let dimensionlist = &dimensionlist &&CODELIST&i..;
					%end;
				%end;

				%let HDVARSEL_INPUT = &dimensionlist;
				%put &HDVARSEL_INPUT.;

			      %let FILECNT = %SYSFUNC(COUNTW(&HDVARSEL_INPUT.));
			      %do i = 1 %to &FILECNT. ; 
				      
			        %global CODELIST&i. ;
			        %let CODELIST&i. = %scan(&HDVARSEL_INPUT., &i.) ;
				    
			        proc sort data = temp3 ;
				        by patid studyclass;
			        run ;

			        proc sort data = DPLocal.&&CODELIST&i.. out = &&CODELIST&i..(drop=indexdt) ;
				        by patid group;
			        run ;
				    
			        data &&CODELIST&i.. ;
			          merge temp3 (in = a keep = patid studyclass indexdt)
			                &&CODELIST&i.. (in = b rename = (group = studyclass)) ;
			      	  by patid studyclass ;
			          if a AND b and %MS_PeriodsOverlap(period1=IndexDt+coalesce("&HDPSWinfrom.",-99999) IndexDt+coalesce("&HDPSWinTo.",99999),                                             
			                                            period2=ADate);
			        run ;

					*set to missing if no claims;
					%isdata(dataset=&&CODELIST&i..);
					%if %eval(&nobs.=0) %then %do;
						%let codelist&i.=;
					%end;

			      %end ;
				      
			      ** Define macro vars to missing if codelists do not exist ** ;
			      %if (&FILECNT. < 7) %then %do j = %eval(&FILECNT+1) %to 7 ;
			        %global CODELIST&j. ;
			        %let CODELIST&j. = ;
			      %end ;
				    
			%mend get_codes;
		    
			%if %eval("&use_hdps." = "Y") %then %do;
				%get_codes;
			%end ;

			/****************************************************************
				DEFINE DEFAULTS AND CALL HDPS MACROS
			*****************************************************************/

			%let hdpsvariables=;
		    %let hd_VarsSelected=;

		    /** IF <2 EXPOSURE GROUPS WITH NEW USERS IDENTIFIED THEN SKIP HDPS and MATCHING  **/
		    %if &numNewExpgroups ge 2 %then %do;

			%if "&matchanalysis" = "PS" %then %do; /*ps matching*/

		    	/** GET START TIME FOR CREATING HDPS COVARIATES AND RUNNING PS MODEL **/
		      	DATA _NULL_;
		        	temp=DATETIME();
		        	call symput('STHD',temp);
		        	call symput('STDATEHD',put(datepart(temp),date9.));
		        	call symput('STTIMEHD',put(timepart(temp),time4.));
		      	RUN;

   				%if %eval("&use_hdps." = "Y") %then %do;

	  	  			%let hdpsvariables=D0:;

			        %if &covariates_considered =%str(  ) %then %do;
			        	%let default_n = 100; 
			        %end;
			        %else %let default_n = &covariates_considered;

		            %if &covariates_selected=%str( ) %then %do;
			            %let newusernum = 200;
		          	%end;
		         	%else %let newusernum = &covariates_selected;

	     			/** SET DEFAULT NUMBER OF HDPS COVARIATES TO BE THE SAMPLE SIZE (N) 
					    OF NEW USERS OF A STUDY DRUG (WHICHEVER IS THE SMALLEST) **/
				  	proc sql noprint;
			        	select count(*)
		        	 	into :size1-:size2
		         	 	from temp3
		         	 	group by exposure;
		       	 	quit;
					%put &size1 &size2;

			        %let min=%sysfunc(min(&size1.,&size2.));
			        %if &newusernum.>&min. %then %let default_k=&min.; 
			        %else %let default_k=&newusernum.;

			        %if &ranking=%str(  ) %then %do;
			            %let default_rank = EXP_ASSOC; 
			        %end;
			        %else %let default_rank = %upcase(&ranking);

		            %if &zero_cell_corr=%str(  ) %then %do;
		            	%let default_outcome_zero = 1; 
		          	%end;
		          	%else %let default_outcome_zero = &zero_cell_corr;

			        *to use just one PS model call;
			        %macro HDPS;
			        	%ms_HDVariableSelection(var_patient_id 			= patid,
			    								var_exposure 			= exposure,
			    								var_outcome 			= event,
			    								top_n					= &default_n.,
			    								k						= &default_k.,
			    								ranking_method 			= "&default_rank.",
			    								outcome_zero_cell_corr 	= &default_outcome_zero.,
			    								outcome_type 			= 'DICHOTOMOUS',
					                          	input_cohort 			= temp3,
					                         	input_dim1				= &CODELIST1 %IF (&CODELIST1. NE %str()) %THEN %DO ; CODE %END ;,
					                          	input_dim2				= &CODELIST2 %IF (&CODELIST2. NE %str()) %THEN %DO ; CODE %END ;,
					                          	input_dim3				= &CODELIST3 %IF (&CODELIST3. NE %str()) %THEN %DO ; CODE %END ;,
					                          	input_dim4				= &CODELIST4 %IF (&CODELIST4. NE %str()) %THEN %DO ; CODE %END ;,
											  	input_dim5				= &CODELIST5 %IF (&CODELIST5. NE %str()) %THEN %DO ; CODE %END ;,
											  	input_dim6				= &CODELIST6 %IF (&CODELIST6. NE %str()) %THEN %DO ; CODE %END ;,
					                          	input_dim7				= &CODELIST7 %IF (&CODELIST7. NE %str()) %THEN %DO ; CODE %END ;);

				        *Merge selected variables back to HD vars;
				        proc sql noprint undo_policy=none;
					    	create table temp3 as
					        select t3.*,
					               hdps.*
					        from temp3 as t3 left join
					             OUTPUT_FULL_COHORT as hdps
					             on t3.patid =  hdps.patient_id;
				        quit;

				        *copy selected varaible name into a macro variable for the logit;
				        proc contents data=OUTPUT_FULL_COHORT(drop=patient_id) out=SelectedVars noprint; run;

				        proc sql noprint;
				          	select NAME
				            into :hd_VarsSelected separated by ' ' 
				            from SelectedVars;
				    	quit;

						/****** Output varinfo file to dplocal and msoc ******/
						%if %eval(&comp.=1) | %sysfunc(exist(dplocal.&RUNID._VARINFO_&LOOK.))=0 %then %do;
							data dplocal.&RUNID._VARINFO_&LOOK.	
								 msoc.&RUNID._VARINFO_&LOOK.;
								set output_all_vars;
							run;	
						%end;
						%else %do;
							proc append base=dplocal.&RUNID._VARINFO_&LOOK.		
				     			data=output_all_vars force;
							proc append base=msoc.&RUNID._VARINFO_&LOOK.		
				     			data=output_all_vars force;
						%end;

					%mend;
					%HDPS;
			
				%end; *USE_HDPS;

      			*Run PS Model;
			    %ms_runmodel(infile = temp3,
			                 CategoricalVars = &classvars., 
			                 ContinuousVars = &noclassvars.,
			                 HDPSVars=&hd_VarsSelected., 
			                 var_exposure = exposure,
			                 var_patient_id = PatId, 
			                 outfile=DATASET_WITH_PS);

				*Temp3 contains all time periods, restrict to current period;
				%if "&analysis" = "tree" & %eval(&look. ne 1)  %then %do;
					data temp3;
						set temp3(where=(time=&look.));
					run;
				%end;

				*Merge selected variables back to HD vars;
				proc sql noprint undo_policy=none;
		      		create table DATASET_WITH_PS as
				    select t3.*,
				           ps.ps
				    from temp3 as t3 left join
				         DATASET_WITH_PS as ps
				         on t3.patid = ps.patid;
				quit;

		      	data converge2;
			        set converge;
			        if &score_type_3. then model_number = 3; 
			        if &score_type_4. then model_number = 4; 
			        if &score_type_5. then model_number = 5; 
			        t = model_number-2; 
			        if status = 0 then score_model_converged = 1;
			        if status = 1 then score_model_converged = 0;
			        keep t model_number score_model_converged;
		      	run;
      
		      	data fit2;
			        set fit;
			        where label2 = "c";
			        length c_stat 8.;
			        c_stat = input(cvalue2,best.);
			        t = 1;
			        keep t c_stat;
		      	run;
		      
		     	data estimates;
		        	merge converge2 fit2;
		        	by t;
		      	run;    

		      	proc datasets library=work nowarn nolist;
		      		delete converge_all;
		      	quit;

		      	data converge_all;
		      	set converge;
		        	if &score_type_3. then model_number = 3; 
		        	if &score_type_4. then model_number = 4; 
		        	if &score_type_5. then model_number = 5; 
		      	run;

				/** TIMING TO CREATE HDPS COVARIATES AND RUN MODEL**/
			    data _null_;
			        temp=DATETIME();
			        seconds=temp-&STHD.;
			        hours=int(seconds/3600);
			        minutes=int((seconds-hours*3600)/60);
			        seconds2=int((seconds-hours*3600-minutes*60));
			        call symput('STOP',temp);
			        call symput('hours',put(hours,4.0));
			        call symput('minutes',put(minutes,2.0));
			        call symput('seconds',put(seconds2,2.0));
			    run;
			    %global HDPStime;
			    %let HDPStime = &hours. h &minutes. m &seconds. s ;

				/*********************************************************************
					OUTPUT FULL DATASETS TO DPLOCAL
				*********************************************************************/
				%put comp = &comp.;
		     	%put look = &look.;
		     	%put psvar = &psvar.;	  


			  	* Copy dataset with HDPS variables for later use;
			  	data dplocal.&RUNID._DATASET_WITH_PS_&comp._&look.;
			  		set DATASET_WITH_PS;
			  	run;

			 	*Calculate percentile for members in the current look period;
				proc rank data=DATASET_WITH_PS
					out=DATASET_WITH_PS_LOOK
				    groups=&percentiles.;
				  	var PS;
				  	ranks percentile;
				  	where time = &look.;
				run;

				/*dplocal.&RUNID._scores_&comp._&look. temporary dataset for this comp*/
				data dplocal.&RUNID._scores_&comp._&look.(rename=percentile1=percentile);
		      		set DATASET_WITH_PS_LOOK(RENAME = (PS = &psvar.));
		     		by patid;
		      		label TIME = "INTERIM ANALYSIS NUMBER";
					percentile1=strip(put(percentile+1,best.));  *need character variable for risk set;
					drop percentile;
		      	run;

	  			*Get percentiles already calculated in prior looks;
	  			%if %sysfunc(exist(dplocal.&RUNID._scores_&PRIOR)) %then %do;
					proc sort data=dplocal.&RUNID._scores_&PRIOR
		  				out=PS_PRIOR(keep=patid indexdt time percentile);
					  	by patid indexdt time;
				      	where time < &look. and analysisgrp = "&analysisgrp";
					run;
			
		  			proc sort data=DATASET_WITH_PS out=DATASET_WITH_PS_PRIOR(RENAME = (PS = &psvar.));
		  				by patid indexdt time;
		  				where time < &look.;
		  			run;

					data DATASET_WITH_PS;
						merge DATASET_WITH_PS_PRIOR
					  	      PS_PRIOR;
					  	by patid indexdt time;
					run;				  
	
				  	data dplocal.&RUNID._scores_&comp._&look.;
				  		set DATASET_WITH_PS
					   		dplocal.&RUNID._scores_&comp._&look.;
				  	run;

				  	proc sort data=dplocal.&RUNID._scores_&comp._&look.;
						by PatId StudyClass;
				  	run;
				%end;

	  
				/****** Output Estimates file to dplocal and msoc ******/

	      			data &RUNID._estimates_&comp._&look.;
						merge ESTIMATES converge_all;
			        	by MODEL_NUMBER;
						format analysisgrp $40.;
				        length PSTYPE $32. analysisgrp $40;
						analysisgrp ="&analysisgrp.";
				        IF MODEL_NUMBER = 3 THEN PSTYPE = "PREDEFINED";
				        IF MODEL_NUMBER = 4 THEN PSTYPE = "HDPS and PREDEFINED";
				        IF MODEL_NUMBER = 5 THEN PSTYPE = "HDPS ONLY"; 
				        IF PSTYPE ne (" ");
				        KEEP analysisgrp PSTYPE C_STAT SCORE_MODEL_CONVERGED NullModel Reason Status;
			      	run;

					proc sql noprint;
	        			select score_model_converged into :c1
	       				from &RUNID._estimates_&comp._&look.
	        			where score_model_converged>=0;  *this will select the one that was executed;
      				quit;

					%if %eval(&comp.=1) | %sysfunc(exist(dplocal.&RUNID._estimates_&look.))=0 %then %do;
						data dplocal.&RUNID._estimates_&look.	
							 msoc.&RUNID._estimates_&look.;
							set &RUNID._estimates_&comp._&look.;
						run;	
					%end;
					%else %do;
						proc append base=dplocal.&RUNID._estimates_&look.	
			     			data=&RUNID._estimates_&comp._&look. force;
						proc append base=msoc.&RUNID._estimates_&look.	
			     			data=&RUNID._estimates_&comp._&look. force;
					%end;

			%end; /* end of ps creation*/

			%if "&matchanalysis" = "MS" %then %do; /* creating score data for MFM*/

	 			%let c1 = 0; /*dummy variable*/

				data DATASET_WITH_PS (drop = char:);
					set temp3;
			 		length charsex $1 charyear $4 charagegroup $2;

			 		*initialize;
			 		charsex = ""; charyear = ""; charagegroup = "";

			   		%let n = %sysfunc(countw(&matchvars.));
					%do i = 1 %to &n.;
					  %if %sysfunc(scan(&matchvars., &i.)) = SEX %then %do;	
						if sex = "F" then charsex = "1";	
				 			else if sex = "M" then charsex = "2";
				 			else charsex = "3";
					  %end;
					  %if %sysfunc(scan(&matchvars., &i.)) = AGEGROUP %then %do;	
						charagegroup = put(agegroupnum,2.0);
					  %end;
					  %if %sysfunc(scan(&matchvars., &i.)) = YEAR %then %do;	
						charyear = compress(year);
					  %end;
				   %end;

					&psvar. = INPUT(CATS(charsex,charagegroup,charyear),10.);
				run;

				data dplocal.&RUNID._DATASET_WITH_PS_&comp._&look.;
					set DATASET_WITH_PS;
				run;

			 	DATA dplocal.&RUNID._scores_&comp._&look.;
			    	SET DATASET_WITH_PS;
			      	by patid;
			      	LABEL TIME = "INTERIM ANALYSIS NUMBER";
			    RUN;

			%end; /*score data for MFM*/

			/*********************************************************************
			 	RUN FULL AND SUBGROUP MATCHING 
			 *********************************************************************/

			/** CLEAN UP**/
		    proc datasets lib = WORK nowarn nolist kill; 
			quit;

			*Initialize macro variables for full analysis case;
		    %global numcat;
	      	%let numcat=0;   *Number of subcategories to analyse for Full analysis;
	      	%let COVARNUM=0; *0 is for Overall analysis;

      		/** SELECT PATIENTS IDENTIFIED IN THIS MONITORING PERIOD **/
     		data TEMP&comp._&look;
        		set dplocal.&RUNID._scores_&Comp._&look.;        
		        where TIME = &look;
		        DUM0=1;
		        MatchedinFull=1;
		        format cat $20.;
		        cat="Overall";
			run;

			/**Find out if subgroup analyses are required for this comp order**/
	      	%let NumSub=0;

      		%ISDATA(dataset=infolder.&ANALYTICSUBGROUPS.);
      		%if %eval(&NOBS.>=1) %then %do;		    

		        *How many subgroup analyses for this comp?;
		        data _SubComp;
			        set infolder.&ANALYTICSUBGROUPS.;
		        	where lowcase(analysisgrp) eq lowcase("&analysisgrp");       
		        run;

		        %ISDATA(dataset=_SubComp);
		        %let NumSub=&NObs.;
		        %put Number of Subgroups for comp &comp.: &NumSub.;
			%end;

			/***************************************************
				Loop through Matching (overall, then subgroup)
			****************************************************/

      		%do Sub=0 %to &NumSub.;  *Note: 0 is for full analysis;

		        /** GET START TIME FOR MATCHING **/
		        data _null_;
		        temp=DATETIME();
			        call symput('STMATCH',temp);
			        call symput('STDATEM',put(datepart(temp),date9.));
			        call symput('STTIMEM',put(timepart(temp),time4.));
		        run;

				%if %eval(&Sub.>=1) %then %do;  *processing a subgroup analysis;

          			*what are the covarnum and categorization to be applied;
          			data _null_;
			        	set _SubComp;
				        if _N_ = &Sub.;
					        call symputx("COVARNUM",COVARNUM);
					        call symputx("CATEGORIZATION",CATEGORIZATION);
					        NumCat=countw(CATEGORIZATION," ");
					        call symputx("NumCat",NumCat);
							call symputx("MatchedinFull",upcase(MatchedinFullOnly));
          			run;
         			%put &COVARNUM. &CATEGORIZATION. &NumCat.;

         			*Populate macro variables required for subgroup analysis;
          			%IF (&COVARNUM. eq %eval(1000)) %THEN %DO; * Sex subgroup;
			        	data _null_;
			        		call symputx("var","Sex");
			        	  	call symputx("CATEGORIZATION","M F O");
			        	  	call symputx("NumCat","3");
			            run;
			        	data TEMP&comp._&look;  
			                set TEMP&comp._&look;
			        		cat=Sex;
			        	run;
			            %put &COVARNUM. &CATEGORIZATION. &NumCat.;
			        %END; *COVARNUM = 1000;
					%IF (&COVARNUM. eq %eval(1001)) %THEN %DO;  * Age subgroups;
						*Extract birth_date;
			            proc sql noprint;
				            create table temp4(drop=cat) as
				            select b.*,
				                   d.birth_date
				            from TEMP&comp._&look as b,
				                 indata.&demtable. as d
				            where b.PatId = d.PatId
							      order by b.patid;
			            quit;

			            *assigning AgeGroup categories;
			            %ms_agestrat(infile=temp4,
			                         outfile=TEMP&comp._&look,
			                         startdt=birth_date,
			                         enddt=indexdt,
			                         timestrat=&CATEGORIZATION.);

						proc sql noprint;
						   alter table TEMP&comp._&look drop birth_date;
						quit;

			            proc datasets library=work nowarn nolist;
			            MODIFY TEMP&comp._&look;  
			            RENAME AgeGroup=Cat;
			            DELETE temp4;
			            quit; 

		        		data _null_;
		        			call symputx("var","age_cat");
		       			run;
					%END; *COVARNUM = 1001;
          			%IF (&COVARNUM. gt %eval(1001) & &COVARNUM. le %eval(1011)) %THEN %DO;
           				%let var=;
			            data _null_;
			            if &COVARNUM. = 1002 then call symputx("var","Year");
			            if &COVARNUM. = 1003 then call symputx("var","Time");
			            if &COVARNUM. = 1004 then call symputx("var","Comorbidscore");
			            if &COVARNUM. = 1005 then call symputx("var","NumIP");
			            if &COVARNUM. = 1006 then call symputx("var","NumIS");
			            if &COVARNUM. = 1007 then call symputx("var","NumED");
			            if &COVARNUM. = 1008 then call symputx("var","NumAV");
			            if &COVARNUM. = 1009 then call symputx("var","NumOA");
			            if &COVARNUM. = 1010 then call symputx("var","NumRx");
			            if &COVARNUM. = 1011 then call symputx("var","NumGeneric");
			            run;
            			%put &var.;

			            %ms_CreateNumCat(infile=TEMP&comp._&look,
			                             outfile=TEMP&comp._&look,
			                             var=&var.,
			                             catString=&CATEGORIZATION.);

					%END; *1001 < COVARNUM <= 1011;
   					%IF (&COVARNUM. eq %eval(1012)) %THEN %DO; * Race subgroup;
            			data _null_;
			        		call symputx("var","race");
			        	 	call symputx("CATEGORIZATION","0 1 2 3 4 5");
			        	  	call symputx("NumCat","6");
			            run;
			        	data TEMP&comp._&look;  
			            	set TEMP&comp._&look;
			        		cat=race;
			        	run;
			            %put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END; *COVARNUM = 1012;
   					%IF (&COVARNUM. eq %eval(1013)) %THEN %DO; * hispanic subgroup;
            			data _null_;
				       		call symputx("var","Hispanic");
				        	call symputx("CATEGORIZATION","Y N U");
				        	call symputx("NumCat","3");
            			run;
		        	  	data TEMP&comp._&look;  
		                	set TEMP&comp._&look;
		        			cat=Hispanic;
		        	  	run;
           				%put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END; *COVARNUM = 1013;
					%IF (&COVARNUM. eq %eval(1014)) %THEN %DO; * pre-post ind subgroup;
            			data _null_;
				       		call symputx("var","prepostind");
				        	call symputx("CATEGORIZATION","NONE PRETERM POSTTERM TERM");
				        	call symputx("NumCat","4");
            			run;
		        	  	data TEMP&comp._&look;  
		                	set TEMP&comp._&look;
		        			cat=prepostind;
		        	  	run;
           				%put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END; *COVARNUM = 1014;
					%IF (&COVARNUM. lt %eval(1000)) %THEN %DO; * Other covariates;	
           				%let CovarVar=COVAR&COVARNUM.;	
            			%put &CovarVar.;
			        	data _null_;					
			        		call symputx("var","&CovarVar.");
			        	  	call symputx("CATEGORIZATION","0 1");
			        	  	call symputx("NumCat","2");
			           	run;

            			data TEMP&comp._&look;  
            				set TEMP&comp._&look(drop=cat);	
        	  				format cat $20.;	
        	  				cat=strip(put(&CovarVar.,best.));
        	 			run;
            			%put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END;	*COVARNUM < 1000;
   					%IF (&COVARNUM. eq %eval(2000)) %THEN %DO; * matchmethod subgroup;
            			data _null_;
				       		call symputx("var","MatchMethod");
				        	call symputx("CATEGORIZATION","BC RE SI LA OT N1 N2 N3 NA");
				        	call symputx("NumCat","9");
            			run;
		        	  	data TEMP&comp._&look;  
		                	set TEMP&comp._&look;
		        			cat=MatchMethod;
		        	  	run;
           				%put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END; *COVARNUM = 2000;
   					%IF (&COVARNUM. eq %eval(2001)) %THEN %DO; * birth type subgroup;
            			data _null_;
				       		call symputx("var","birth_type");
				        	call symputx("CATEGORIZATION","0 1 2 3 4 5 8 9");
				        	call symputx("NumCat","8");
            			run;
		        	  	data TEMP&comp._&look;  
		                	set TEMP&comp._&look;
		        			cat=strip(put(birth_type,best.));
		        	  	run;
           				%put &COVARNUM. &CATEGORIZATION. &NumCat.;
          			%END; *COVARNUM = 2001;


			/***************************************************************************/
			/** MATCH WITHIN MONITORING PERIOD                                        **/
			/** KEEP PRIOR MATCHES                                                    **/
			/***************************************************************************/

          			*assigning generic names to subgroups;
         			data TEMP&comp._&look;
          				set TEMP&comp._&look(drop=dum:);
          					dum0=0;
					        array DUM{*} DUM1-DUM&NumCat.;
					        do SubComp=1 to &NumCat.;
					        	if scan("&CATEGORIZATION.",SubComp,' ')=Cat then dum(SubComp)=1; else dum(SubComp)=0; 
					        end;
					        SubComp=SubComp-1;
					run;

          			* For subgroup analyses, if specified, only use the patients that were matched in the overall 
           			  analysis (not the entire population that includes folks that couldnt be matched).;
          			%ISDATA(dataset=MatchedInFullAnalysis);
          			%IF %EVAL(&NOBS.=0) %THEN %DO;
            		proc sort nodupkey data=dplocal.&RUNID._matched_&Comp._&look. 
                                	   out=MatchedInFullAnalysis(keep=Patid);
		            	by PatId;
           				where missing(matchid)=0;
           			run;
         			%end;

			        data TEMP&comp._&look;
			        	merge TEMP&comp._&look(in=a)
			            	  MatchedInFullAnalysis(in=b);
			          	by PatId;

          				*the variable MatchedinFull was inintialized to 1 in TEMP&comp._&look;
         				if upcase("&MatchedInFull.")="Y" then do;
                    		if a and b then MatchedinFull=1; 
                    		else MatchedinFull=0; 
          				end;
          			run;

				%end;*sub>=1;

        		* For full analysis, only process cat=0 (here %eval(1-(&sub.=0))) with NumCat=0
         	  	  Otherwise, start at 1 and then loop until the &NumCat.>=1;
				%do Cat=%eval(1-(&sub.=0)) %to &NumCat.; 

          			*select subgroup data from those that have been matched in full analysis;
		        	data  TEMP&comp._&look._&cat.;
			        	set TEMP&comp._&look;
			          	if Dum&cat.=1 and MatchedinFull=1; 
			        run;

	          		%ISDATA(dataset=TEMP&comp._&look._&cat.);
	          		%IF %EVAL(&NOBS.>0) %THEN %DO;

			        	proc datasets library=work nowarn nolist;
			          		delete Matchlist_1;
			          	quit;    

						*Matching;
						%ms_starttimer(timestart=matchruntime);
			            %ms_NearestNeighborMatch(InFile=TEMP&comp._&look._&cat.,
	                                     		 OutFile=Matchlist_1,
	                                    		 MatchVars=Patid,
	                                   			 PSVar=&psvar,
	                                     		 MatchRatio=&ratio,
	                                     		 WithReplacement=N,
	                                     		 Caliper=&caliper,
										 		 MatchAnalysis=&matchanalysis.);
						%ms_stoptimer(timestart=matchruntime);
						%ms_outputruntimes(timevar=matchruntime, step=%str(Nearest Neighbor Matching: sub=&sub. cat=&cat.), group=&analysisgrp., monitoringperiod=&look.);

	            		data Matchlist_1_new(rename=match_distance2=match_distance);
	            			set Matchlist_1;
	           			 	by Patid;

	            			if first.patid then do;
	              				MatchNumber=0;
	              				match_distance2=.;
	              				ps=TRT_PS;
	              				output;
	            			end;

	            			patid=ctrl_Patid;
	            			ps=CTRL_PS;
	            			match_distance2=match_distance;
	            			output;

	            			keep Patid MatchNumber set_num ps match_distance2;
	            		run;

			            proc sql noprint undo_policy=none;
			            	create table Matchlist_1 as
			            	select base.*,
			                	   mtch.MatchNumber,
			                  	   mtch.set_num,
			                   	   mtch.ps, 
			                       mtch.match_distance
			            	from TEMP&comp._&look._&cat. as base
			           		inner join Matchlist_1_new as mtch
			            	on base.Patid = mtch.Patid
			            	order by Patid;
			            quit;

	          			/** IF AT LEAST ONE MATCH FOUND THEN DO **/
	          			%if %sysfunc(exist(Matchlist_1)) %then %do;
			            	data &psvar.Matchlist_1;
			              		set Matchlist_1;
			              		length matchID $50.;

			              		matchID  = COMPRESS(strip(put(TIME,best.))||strip(put(SET_NUM,best.)));
			             		label matchID= "MatchID nearest neighbor caliper &caliper ratio &ratio.";
			              		keep patid matchID;
			            	run;    
	          			%end;

	          			/** IF NO MATCHES FOUND THEN DO **/
	         			%else %do;
				            data &psvar.Matchlist_1;
				            	set TEMP&comp._&look._&cat. (KEEP =patid);
				              	length  matchID $50.;
				              	matchID = " ";
			             		label matchID= "MatchID nearest neighbor caliper &caliper ratio &ratio.";
				              	keep patid matchID;
				            run; 
	         			%end;

						*Create dummy formated output file when no matches;
						%put &match_Error.;
            			%if %eval(&match_Error.>=1) %then %do;
               				%put WARNING: No matches were found for Sub=&sub. and Cat=&Cat.;
               				data matchds_0;
               					set TEMP&comp._&look._&cat.(keep=PatId);
				                	format matchID $50.; 
				                  	matchID='';
               				run;
            			%end;

            			%else %do;
               				proc sort data = &psvar.Matchlist_1 out=matchds_0; 
                				by patid; 
                			run;
            			%end;

	          		%end; 
	          		%else %do;  *no obs in TEMP&comp._&look._&cat.;
	             		data matchds_0;
	             			set TEMP&comp._&look(obs=0 keep=PatId);
	                			format matchID $50.; 
	                			matchID='';
	             		run;
	          		%end;

		          	*Square with category;
			        data matchds_0(rename=cat2=cat);
			        	merge matchds_0(in=a)
			                  TEMP&comp._&look.(where=(Dum&cat.=1) keep=PatId Cat Dum&cat.);
			          	by PatId;
			          	format cat2 $20. covarnum 4.;
			          	cat2=cat;
			          	covarnum=&COVARNUM.;
			          	drop cat Dum&cat.;
		          	run;

	          		%ISDATA(dataset=matchds_0);
	          		%if %eval(&nobs.=0) %then %do; *Empty - one empty line added to show it ran, but empty;
			            data cat;
				            format cat $20. covarnum 4.;
				            Cat = scan("&CATEGORIZATION.", &cat., ' ') ;
				            covarnum=&COVARNUM.;
			            run;

			            data matchds_0;
			            set matchds_0
			                cat;
			        	run;
					%end; 

					*NOTE: the "_" prefix is for the accumulating file;
	          		%if %eval(&cat.>=2) %then %do;  
	            		data _matchds;
	            			set _matchds
	                			matchds_0;
	            			by PatId;
	            		run; 
		          	%end;
		          	%else %do;
		             	data _matchds;
		             		set matchds_0;
		             	run; 
		          	%end;

	          		proc datasets library=work nolist nowarn;
	          			delete matchds: TEMP&comp._&look._&cat. cat &psvar.Matchlist_1; 
	          		quit;

				%end; * cat;

        		%if &sub.=0 %then %do;
          			*Squaring and adding variables in scores;
					data dplocal.&RUNID._matched_&comp._&LOOK.;
		            	merge _matchds (rename=Cat=StratumName)
		                	  dplocal.&RUNID._scores_&Comp._&look.(where=(time=&look.));*All vars including PS;
		            	by patid;
		            	format AllPts $50.;
		            	AllPts="1";*required for creating risk sets for analyses unadjusted for Matchid;
		          	run; 
        		%end;
        		%else %do;
          			*adding variables from scores;
          			data _matchds0;
            			merge _matchds(in=a)
                  			  dplocal.&RUNID._scores_&comp._&look.(in=b where=(time=&look.));
			            by patid;
			            if a;
			            format AllPts $50.;
			            AllPts="1";
          			run; 

          			*If no match was possible among the patients, reset to empty line;
					%ISDATA(dataset=_matchds0);
          			%if %eval(&NOBS.=0) %then %do;
	            		data _matchds0;
	            			set _matchds;
	            		run;
					%end;

		          	*need to set instead of append to avoid missing variable warns;  
		           	data dplocal.&RUNID._matched_&comp._&LOOK.;
		           		set dplocal.&RUNID._matched_&comp._&LOOK.
		              	    _matchds0 (rename=Cat=StratumName);*only incremental patients are added here;
		           		by Patid;
		           	run;

	        	%end;

	       		proc datasets nowarn nolist;
	        		delete _matchds:;
	        	quit;

	        	* if a prior look exists, update vars from scores for those matched in previous look(s) from the same run
	          	  and append newly matched patients (this look=time);
	         
	        	%if %sysfunc(exist(&DPLocalPointer..&RUNID._matched_&PRIOR)) %then %do;

	         		%if &sub.=0 %then %let where=COVARNUM in(0,.); 
	          		%else %let where=COVARNUM=&COVARNUM.;

		          	*Update event and Followuptime for those previously matched; 
		          	*Percentiles calculated using reestimated PS (regression is run on all patients);
					proc sort data=&DPLocalPointer..&RUNID._matched_&PRIOR. out=_Tempo1 %if "&matchanalysis" eq "PS" %then (drop= &hdpsvariables.);;
						by PatId IndexDt StudyClass;
						where lowcase(analysisgrp) = lowcase("&analysisgrp");
					run;

				  	proc sort data=&INDATA._&look. out=_Tempo2(rename=Group=StudyClass);
				  		by PatId IndexDt Group;
				  	run;

				  	proc sort data=dplocal.&RUNID._DATASET_WITH_PS_&comp._&look. out=_Tempo3;
			 			by PatId IndexDt StudyClass;
			  		run;

			  		data PrevMatched;
						merge _Tempo1(in=a where=(time<&look. and &where.)) /*Only this covarnum*/
			                  _Tempo2(keep=StudyClass PatId IndexDt EventDt LastLook: &Followuptime. Event) /*all patients*/
							  _Tempo3(keep=StudyClass PatId IndexDt &hdpsvariables.); /*all patients*/
			          	by PatId IndexDt StudyClass;
			          	if a and missing(PatId)=0;
			          	format AllPts $50.;
			          	AllPts="1";*required for creating risk sets for analyses unadjusted for Matchid;
					run;

			        %ISDATA(dataset=PrevMatched);
	          		%IF %EVAL(&NOBS.>0) %THEN %DO;
			            *Adding row for patients identified in prior look(s) to patient identified in this look;
			            *Note that Matchid and PS are carried forward from the past;
			            *using a data step method instead of proc append to avoid numerous w.a.r.n.i.n.g.s. from 
			             the arbitrary HDPS variables names across looks;
						data dplocal.&RUNID._matched_&comp._&LOOK.;
			           		set dplocal.&RUNID._matched_&comp._&LOOK.
			                	PrevMatched;
			            	by Patid;
						run;
					%END;
	  
			  		proc sort nodupkey data=dplocal.&RUNID._matched_&comp._&LOOK.;
			  			by _all_;
			  		run;

	         		proc datasets library=work nolist nowarn;
	          			delete PrevMatched;
	          		quit;

	        	%end;  *&DPLocalPointer..&RUNID._matched_&comp._&PRIOR exist;

		        /** TIMING TO RUN MATCHING **/
		        DATA _NULL_;
		          temp=DATETIME();
		          seconds=temp-&STMATCH.;
		          hours=int(seconds/3600);
		          minutes=int((seconds-hours*3600)/60);
		          seconds2=int((seconds-hours*3600-minutes*60));
		          call symput('STOP',temp);
		          call symput('hours',put(hours,4.0));
		          call symput('minutes',put(minutes,2.0));
		          call symput('seconds',put(seconds2,2.0));
		        RUN;
		        
		        %global matchingtime;
		        %let matchingtime = &hours. h &minutes. m &seconds. s ;
		        %put &matchingtime.;


				/**** Attrition level 4 ****/
				*temporary dataset b/c do not want to exclude from actual data;
				data _tempattrition;
					set dplocal.&RUNID._matched_&comp._&LOOK.;
					where covarnum = 0;
				run;

		     	/*5*/	%matchingattrition(file=_tempattrition,ToExcl=missing(matchID)=1);
				%if "&analysis" ne "tree" %then %do;
		     	/*6*/	%matchingattrition(file=_tempattrition,ToExcl=event=0);
				%end;

				proc datasets nowarn noprint;
					delete _tempattrition;
				quit;


				/*************************************************************/
		    	/** Create Risk sets if analysis is not signal detection    **/
		    	/*************************************************************/

				%if "&analysis" ne "tree" %then %do;
			
					/***************************************************************************/
			    	/** CREATE RISK SETS                                                      **/
			    	/***************************************************************************/

					%if %eval(&sub.=0) %then %do;  * full analysis;

			        	proc datasets library=msoc nolist nowarn;
			          		delete &RUNID._RISKSETDATA_&COMP._&LOOK.
			                 	   &RUNID._RISKDIFFDATA_&COMP._&LOOK.
			                   	   &RUNID._SURVIVALDATA_&COMP._&LOOK.; 
			          	quit;

						%let covarnum=0;
			          	%let Stratvar=Overall;
			          	%let CurrentCat = Overall;

	          			*At least one event;
						proc sql noprint;
				        	select max(EVENT) into :HadEvent
				          	from dplocal.&RUNID._matched_&Comp._&LOOK.
				          	where covarnum=0;
				        quit;

		        		%let var = AllPts;
	          			%let varmiss = AllPts;
	          			%put Creating RiskSet for: &var.;  
	          			%ms_CreateRiskSet(file=DPlocal.&RUNID._matched_&comp._&LOOK,
	                           			  varmiss=&varmiss.,
				                          var=&var.,
				                          covarnum=&COVARNUM.);

				        %let var = matchid;
				        %let varmiss = matchid;
				        %put Creating RiskSet for: &var.;
				        %ms_CreateRiskSet(file=DPlocal.&RUNID._matched_&comp._&LOOK,
				                          varmiss=&varmiss.,
				                          var=&var.,
				                          covarnum=&COVARNUM.);

			 			%if "&matchanalysis" = "PS" %then %do;
				        	%let Stratvar=Percentiles;
				           	%let CurrentCat = Percentiles;
							%let var = percentile;
				           	%let varmiss = percentile;
						   	%put Creating RiskSet for: &var.;
				           	%ms_CreateRiskSet(file=DPlocal.&RUNID._matched_&comp._&LOOK,
				                        	  varmiss=&var.,
				                              var=&var.,
				                              covarnum=&COVARNUM.);
						%end;

	         			/**Unconditional Analysis**/
	       				%if "&unconditional"="y" %then %do;
	          				%let Stratvar=Overall;
	              			%let CurrentCat = Overall Unconditional;
					        %let var =AllPts;
					        %let varmiss = matchid;
					        %put Creating RiskSet for: &var.;
	          				%ms_CreateRiskSet(file=DPlocal.&RUNID._matched_&comp._&LOOK,
	                        			      varmiss=&varmiss.,
	                            			  var=&var.,
	                            			  covarnum=&COVARNUM.);
						%end;

	          			%if %eval(&HadEvent. eq 0) %then %do;
	            			%put "WARNING: no patients in the exposure or comparator cohort have an event";
	          			%end;

		        		%end;
		        		%else %do;* Sub gt 1;

	         				%let Stratvar=&var.;

	          				%do cat=1 %to &numcat.;  *categories within this subgroup;

					            %let CurrentCat = %scan(&CATEGORIZATION., &cat., ' ') ;
					            %put &CurrentCat.;
					            %put &Stratvar.;

					            data ForRisk;
					           		set DPlocal.&RUNID._matched_&comp._&LOOK;
					            	where covarnum=&covarnum. and scan("&CATEGORIZATION.",&cat.,' ')=StratumName;
					            run;

					            %let var = AllPts;
					            %let varmiss = AllPts;
								%put Creating RiskSet for: &var.;
					            %ms_CreateRiskSet(file=ForRisk,
					                              varmiss=&varmiss.,
												  var=&var.,
					                              covarnum=&COVARNUM.);

					            %let var = matchid;
					            %let varmiss = matchid;
					            %put Creating RiskSet for: &var.;
					            %ms_CreateRiskSet(file=ForRisk,
					                              var=&var.,
					                              varmiss=&varmiss.,
					                              covarnum=&COVARNUM.);

					          	/**Unconditional Analysis**/
					      		%if "&unconditional"="y" %then %do;
					            	%let CurrentCat = &CurrentCat. Unconditional;
							        %let var =AllPts;
							        %let varmiss = matchid;
							        %put Creating RiskSet for: &var.;
					          		%ms_CreateRiskSet(file=ForRisk,
					                 	              varmiss=&varmiss.,
					                       		      var=&var.,
					                            	  covarnum=&COVARNUM.);
					      		%end;

					            proc sql noprint;
					            	select max(EVENT) into :HadEvent
					            	from ForRisk;
					            quit;

					            %if %eval(&HadEvent. eq 0) %then %do;
					           		%put WARNING: no patients in the exposure or comparator cohort have an event for covarnum &covarnum. and stratum &var. = &CurrentCat.;
					            %end; *HadEvent;

	      	  				%end;*cat;

				          	proc datasets library=work nowarn nolist;
				          		delete ForRisk;
				          	quit;

						%end;* Subgroup analysis;

					%end; ** Loop through sub;

				%end; /*Create risk set*/

			  	proc datasets library=dplocal nolist nowarn;
		      		delete &RUNID._DATASET_WITH_PS_&comp._&look.;
		      	quit;

			%end; * if &numNewExpgroups ge 2;

			%if &numNewExpGroups<2 %then %do;
    
     			%if &look = 1 or %sysfunc(exist(&DPLocalPointer..&RUNID._matched_&PRIOR)) ne 1 %then %do;
    
			        data dplocal.&RUNID._scores_&comp._&look.;
			          set TEMP3;
			          by patid;
			          label TIME = "INTERIM ANALYSIS NUMBER";
			        run;
		    
			        data dplocal.&RUNID._matched_&comp._&look.;          
			          set dplocal.&RUNID._scores_&comp._&look.;
			          length matchid $50.;
			      		
			          matchid="";
			          &psvar. = .;

			          format cat $20. percentile $12.;
			          cat="Overall";
			          percentile="";
			          StratumName="Overall";
			          COVARNUM=0;
			          _LEVEL_=.;
			        RUN;
    
      			%end;
      
      			%if %sysfunc(exist(&DPLocalPointer..&RUNID._matched_&PRIOR)) %then %do;
			        data dplocal.&RUNID._scores_&comp._&look.;
			        	set TEMP3;
			          	by patid;
			          	label time = "INTERIM ANALYSIS NUMBER";
			        run;
			    
			        data TEMP_matched;
			          set dplocal.&RUNID._scores_&comp._&look.;
			          where TIME = &look;
			          length matchid $50.;
			          matchid="";
			        run;
			    
			        data temp_allmatched;
			          set temp_matched 
			              &DPLocalPointer..&RUNID._matched_&PRIOR(where=(lowcase(analysisgrp)=lowcase("&analysisgrp")));
			        run;
			      	
			        proc sort data = temp_allmatched; 
			        	by patid; 
			        run;
			      	
			        data dplocal.&RUNID._matched_&comp._&LOOK;
			        	MERGE temp_allmatched 
			              	  dplocal.&RUNID._scores_&comp._&look.;
			        	by patid;
			    		format cat $20. percentile $12.;
			    		cat="Overall";
						percentile="";
			    		COVARNUM=0;
			    		_LEVEL_=.;
			        run;
      			%end;  

			%end; *&numexposuregroups<2;
	
			/********************************************/
			/* Save _Matched_ files to DPLOCAL and MSOC */
			/********************************************/
 
			%if %eval(&comp.)=1 | %sysfunc(exist(dplocal.&RUNID._matched_&Look.))=0 %then %do;

				%if %lowcase("&INDLEVEL")="y" %then %do;
				/* Output dataset for MSOC without patient identifiers or covariates (other than subgroup indicators) */
					data msoc.&RUNID._matched_&look.;
			  	    	retain analysisgrp studyclass age_cat sex race hispanic EVENT &FOLLOWUPTIME. matchid;
			  	      	set dplocal.&RUNID._matched_&comp._&LOOK;
			  	      	keep analysisgrp exposure studyclass age_cat sex race hispanic EVENT &FOLLOWUPTIME. matchid &psvar. LastLookFollowed
			  	           COMORBIDSCORE TIME YEAR numIP numED numIS numAV numOA numRX numGeneric numClass &keepcovars. stratumname covarnum &percentile. &t4psavars.;
			  	    run;
				%end;

				data dplocal.&RUNID._matched_&LOOK;
					set dplocal.&RUNID._matched_&comp._&LOOK;
				run;

			%end;
			%else %do;

				%if %lowcase("&INDLEVEL")="y" %then %do;
				/* Output dataset for MSOC without patient identifiers or covariates (other than subgroup indicators) */
					data msoc.&RUNID._matched_&comp._&look.;
			  	    	retain analysisgrp studyclass age_cat sex race hispanic EVENT &FOLLOWUPTIME. matchid;
			  	      	set dplocal.&RUNID._matched_&comp._&LOOK;
			  	      	keep analysisgrp exposure studyclass age_cat sex race hispanic EVENT &FOLLOWUPTIME. matchid &psvar. LastLookFollowed 
			  	           COMORBIDSCORE TIME YEAR numIP numED numIS numAV numOA numRX numGeneric numClass &keepcovars. stratumname covarnum &percentile. &t4psavars.;
			  	    run;

					*Using set b/c HDPS vars could differ by comparison;
					data msoc.&RUNID._matched_&look.;
						set msoc.&RUNID._matched_&look.
						    msoc.&RUNID._matched_&comp._&look.;
					run;

					proc datasets nowarn noprint lib=msoc;
					 delete &RUNID._matched_&comp._&look.;
					quit;	
				%end;

				data dplocal.&RUNID._matched_&LOOK;
				    set dplocal.&RUNID._matched_&LOOK
					    dplocal.&RUNID._matched_&comp._&LOOK;
				run;

			%end;

			/*********************************/
			/* Save _Scores_ file to dplocal */
			/*********************************/

			%if %eval(&comp.)=1 | %sysfunc(exist(dplocal.&RUNID._scores_&Look.))=0 %then %do;
				data dplocal.&RUNID._scores_&look.;
					set dplocal.&RUNID._scores_&comp._&look.;
				run;
			%end;
			%else %do;

				*Using set b/c HDPS vars can differ between scenarios;
			  	data dplocal.&RUNID._scores_&look.;
				    set dplocal.&RUNID._scores_&look.
					    dplocal.&RUNID._scores_&comp._&look.;
				run;
			%end;

			%if "&analysis." ne "tree" %then %do;

				/********************************/
				/* Save _RiskDiff_ file to MSOC */
				/********************************/

				%if %eval(&comp.)=1 | %sysfunc(exist(msoc.&RUNID._riskdiffdata_&Look.))=0 %then %do;
					data msoc.&RUNID._riskdiffdata_&look.;
						set msoc.&RUNID._riskdiffdata_&comp._&look.;
					run;
				%end;
				%else %do;
					proc append base=msoc.&RUNID._riskdiffdata_&look. data=msoc.&RUNID._riskdiffdata_&comp._&look. force;		
				%end;

				%if %eval(&type.=2) %then %do;

					/*******************************/
					/* Save _RiskSet_ file to MSOC */
					/*******************************/

					%if %eval(&comp.)=1 | %sysfunc(exist(msoc.&RUNID._risksetdata_&Look.))=0 %then %do;
						data msoc.&RUNID._risksetdata_&look.;
							set msoc.&RUNID._risksetdata_&comp._&look.;
						run;
					%end;
					%else %do;
						proc append base=msoc.&RUNID._risksetdata_&look. data=msoc.&RUNID._risksetdata_&comp._&look. force;		
					%end;

					/********************************/
					/* Save _Survival_ file to MSOC */
					/********************************/

					%if %eval(&comp.)=1 | %sysfunc(exist(msoc.&RUNID._survivaldata_&Look.))=0 %then %do;
						data msoc.&RUNID._survivaldata_&look.;
							set msoc.&RUNID._survivaldata_&comp._&look.;
						run;
					%end;
					%else %do;
						proc append base=msoc.&RUNID._survivaldata_&look. data=msoc.&RUNID._survivaldata_&comp._&look. force;		
					%end;

				%end; /*Type 2 risk set data*/

				*Remove comp specific datasets;
				proc datasets nowarn noprint lib=msoc;
					delete &RUNID._risksetdata_&comp._&look. &RUNID._riskdiffdata_&comp._&look. &RUNID._survivaldata_&comp._&look.;
				quit;

			%end; /*Risk set data*/

			/********************/
			/* Output Attrition */
			/********************/
			data dplocal._matchattrition;
				format analysisgrp eoi ref $40. descr $500.;
				set dplocal._matchattrition;
	    		retain analysisgrp eoi ref level descr eoi_remaining eoi_excluded ref_remaining ref_excluded;

				analysisgrp = "&analysisgrp.";
				eoi = "&eoigrp";
				ref = "&refgrp";
				lastremain1=lag(eoi_remaining);
				lastremain0=lag(ref_remaining);

				eoi_excluded = LastRemain1 - eoi_remaining;
				ref_excluded = lastremain0 - ref_remaining;

   				if level=1 then descr="Patients identified in EOI and REF cohorts";
   				if level=2 then descr="Patients excluded due to same day EOI and REF initiation";
   				if level=3 then descr="Patients excluded due to earlier initiation of EOI of REF";
   				if level=4 then descr="Patients excluded due to earlier initiation of EOI of REF in a prior look";
					%if "&analysis" ne "tree" %then %do;
					if level=4 then descr="";
					%end;	
   				if level=5 then descr="Patients in matched cohort";
   				if level=6 then descr="Events for patients in matched cohort";
				drop lastremain:;
			run;

			*Save to MSOC folder;
			%if %eval(&comp.)=1 | %sysfunc(exist(msoc.&runid._match_attrition_&look.))=0 %then %do;
				data msoc.&runid._match_attrition_&look.;
					set dplocal._matchattrition;
				run;
			%end;
			%else %do;
				proc append base=msoc.&runid._match_attrition_&look. data=dplocal._matchattrition force;		
			%end;

			proc datasets nowarn noprint lib=dplocal;
				delete _matchattrition;
			quit;

			/***********************************
			 CLEAN UP AND CREATE OUTPUT TABLES
			***********************************/

		    ** Clean up work directory before next comp	** ;
		    proc datasets lib= work nowarn nolist kill; quit; 

			*Produce PS matched Table 1s;
    		%ms_matchtables(comp = &comp., look = &look., psvar=&psvar., caliper=&caliper., input_file = infolder.&comparison.);	

			proc datasets nowarn noprint lib=dplocal;
				delete &RUNID._matched_&comp._&look. &RUNID._scores_&comp._&look.;
			quit;

  			%continue:

			/*Output run time*/
			%ms_stoptimer(timestart=covadjustruntime);
			%ms_outputruntimes(timevar=covadjustruntime, step=%str(Covariate Adjustment), group=&analysisgrp., monitoringperiod=&look.);

		%end ; ** Loop through analyssigrps ;

	%nextlook:
	%end ; ** Loop through looks ;

	%exit:

	%put NOTE: ********END OF MACRO: ms_covariateadjustment v4.0 ********;

%mend ms_covariateadjustment;
