****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_treeconvertchildparent.sas  
*
* Created (mm/dd/yyyy): 06/30/2019
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
* 	Convert a vertical tree in child-parent format to dx-node format                                   
*   
*
*  Program inputs:   
*	- infolder.&treelookup
*                                    
*  Program outputs:  
*   - expandedtree
*                                                                                                  
*  PARAMETERS:     
*
* 
*  Programming Notes:                                                                                                                                                           
*	- requires input treefile in a two-column dataset with Child and Parent variables 														
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_treeconvertchildparent();
		
	%put =====> MACRO CALLED: ms_treeconvertchildparent v1.0;

	/*Extract each level of the tree*/

	*1st level - no parent;
	data leaf1
		 restoftree;
		set infolder.&treelookup.;

		id = _n_; /*row identifier*/

		if missing(parent)=1 then output leaf1;
		else output restoftree;
	run;

	%let leaflevel = 2;

	*Determine length of node;
	proc sql noprint;
		select max(length(child), length(parent)) into: treelength
		from infolder.&treelookup.;
	quit;

	/*Extract additional levels if they exist*/
	%isdata(dataset=restoftree);
	%if %eval(&nobs.>0) %then %do;

		*loop through until no more children;
		%let rc = 1;
		%do %until (&rc.=0);
			proc sql noprint;
				create table leaf&leaflevel. as
				select *
				from restoftree
				where parent in (select child as parent from leaf%eval(&leaflevel.-1));
			quit;
			
			*remove rows from restoftree;
			proc sql noprint undo_policy=none;
				create table restoftree as
				select *
				from restoftree
				where id not in (select id from leaf&leaflevel.);
			quit;

			%isdata(dataset=restoftree);
			%if %eval(&nobs.>0) %then %do;
				%let leaflevel = %eval(&leaflevel.+1);
			%end;
			%else %do;
				/*stop looping*/;
				%let rc = 0;
			%end;
		%end;

		*Merge across all levels to build out full tree;
		proc sql noprint;
			create table fulltree as
			select 
				/*last leaf*/
				  leaf&leaflevel..child as dx
				, leaf&leaflevel..parent as parent&leaflevel.
				%do leaf = %eval(&leaflevel.-1) %to 2 %by -1;
				 , leaf&leaf..parent as parent&leaf.
				%end;
			from leaf&leaflevel. as leaf&leaflevel.
			%do leaf = %eval(&leaflevel.-1) %to 2 %by -1;
			left join leaf&leaf. as leaf&leaf. on leaf&leaf..child = leaf%eval(&leaf.+1).parent
			%end;
			;
		quit;

	%end; /*additional levels*/
	%else %do;
		*number of levels in tree;
		%let leaflevel = %eval(&leaflevel-1); /*Equals 1*/
	%end;

	/*Always 1 level of leaf*/
	data expandedtree(keep=dx_codetype dx node level);
		format dx $11. dx_codetype $3. node $&treelength..;			
		set leaf&leaflevel.;

		dx_codetype="09";
		dx=child;
		node = child;
		level = &leaflevel.;
	run;

	*Add additional levels;
	%do leaf = %eval(&leaflevel.) %to 2 %by -1;
		data expandedtree(keep=dx_codetype dx node level);
			set expandedtree fulltree(in=a keep=dx parent&leaf.);
			if a then do;
				dx_codetype="09";
				node = parent&leaf.;
				level = %eval(&leaf.-1);
			end;
		run;
	%end;

	*clean up;
	proc datasets nowarn noprint lib=work;
		delete leaf: restoftree fulltree;
	quit;

	%put NOTE: ******** END OF MACRO: ms_treeconvertchildparent v1.0 ********;

%mend ms_treeconvertchildparent;
